# ---------------------------------------------------------------------------
# The contents of this file are subject to the JOnAS Public License Version 
# 1.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License on the JOnAS web site.
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.
# See the License for the specific terms governing rights and limitations
# under the License.
#
# The Original Code is JOnAS application server code released July 1999.
#
# The Initial Developer of the Original Code is Bull S.A.
# The Original Code and portions created by Bull S.A. are
#    Copyright (C) 1999 Bull S.A. All Rights Reserved.
#
# Contributor(s): ______________________________________.
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

TEMPLATE=$JONAS_ROOT/bin/template

# --------------------------------------------------------
# Get args
# no arg => interactive mode.
# --------------------------------------------------------
BNAME=
TYPE=
PACKAGE=
PK=
if [ $# -gt 0 ]
then
    BNAME=$1
fi
if [ $# -gt 1 ]
then
    TYPE=$2
fi
if [ $# -gt 2 ]
then
    PACKAGE=$3
fi
if [ $# -gt 3 ]
then
    PK=$4
fi

# --------------------------------------------------------
# Get bean caracteristics
# --------------------------------------------------------
if [ -z "$BNAME" ]
then
    echo "Choose the bean name, starting with a capital letter."
    echo "Avoid the reserved names : Home, EJB, Session, Entity."
    echo "bean name : \c"
    read BNAME
fi

if [ -z "$TYPE" ]
then
    echo "Type of bean ?"
    echo "ec = Entity CMP"
    echo "eb = Entity BMP"
    echo "sl = Session Stateless"
    echo "sf = Session StateFul"
    echo "Your choice: \c"
    read TYPE
fi

if [ -z "$PACKAGE" ]
then
    echo "Enter the dot separated package name, usually matching"
    echo "the current directory, where the beans will be compiled."
    echo "Package Name: (dot separated name: x.y.z) \c"
    read PACKAGE
fi

case $TYPE in
e*)
    if [ -z "$PK" ]
    then
	case $TYPE in
	e*)
	    echo "Choose the PK Class name."
	    echo "s = java.lang.String"
	    echo "i = java.lang.Integer"
	    echo "o = deferred to deployment time"
	    read PK
	    ;;
	esac
    fi
    ;;
s*)
    PK=none
    ;;
esac

# --------------------------------------------------------
# Build PK Class Name
# --------------------------------------------------------
case $PK in
s)
    PKCLASS=java.lang.String
    ;;
i)
    PKCLASS=java.lang.Integer
    ;;
o)
    PKCLASS=java.lang.Object
    ;;
*)
    PKCLASS= 
    ;;
esac

export BNAME TYPE PACKAGE PKCLASS

echo "Creating bean $BNAME (type $TYPE) in package $PACKAGE"

# --------------------------------------------------------
# Build Standard Deployment Descriptor file
# --------------------------------------------------------
FILE=${BNAME}.xml
>$FILE

case $TYPE in
sl)
	cat $TEMPLATE/Stateless.xml | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
sf)
	cat $TEMPLATE/Stateful.xml | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
ec)
	cat $TEMPLATE/CMP.xml | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@PKCLASS@;$PKCLASS;" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
eb)
	cat $TEMPLATE/BMP.xml | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@PKCLASS@;$PKCLASS;" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
esac

# --------------------------------------------------------
# Build JOnAS specific  Deployment Descriptor file
# --------------------------------------------------------
FILE=jonas-${BNAME}.xml
>$FILE

case $TYPE in
s*)
	cat $TEMPLATE/jonas-Session.xml | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@BNAME@;$BNAME;g" >> $FILE
	;;
ec)
	cat $TEMPLATE/jonas-CMP.xml | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
eb)
	cat $TEMPLATE/jonas-BMP.xml | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
esac

# --------------------------------------------------------
# Build remote interface file
# --------------------------------------------------------
FILE=${BNAME}.java
>$FILE

cat $TEMPLATE/Remote.java |
sed "s;@BNAME@;$BNAME;g" |
sed "s;@PACKAGE@;$PACKAGE;" >>$FILE

# --------------------------------------------------------
# Build implementation file
# --------------------------------------------------------
FILE=${BNAME}Bean.java
>$FILE

case $TYPE in
sl)
	cat $TEMPLATE/Stateless.java |
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Session.java |
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	;;
sf)
	cat $TEMPLATE/Stateful.java |
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Session.java |
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	;;
eb)
	cat $TEMPLATE/Entity.java |
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PKCLASS@;$PKCLASS;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Bmp.java |
	sed "s;@PKCLASS@;$PKCLASS;g" |
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	;;
ec)
	cat $TEMPLATE/Entity.java |
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PKCLASS@;$PKCLASS;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Cmp.java |
	sed "s;@PKCLASS@;$PKCLASS;" |
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	;;
esac

cat $TEMPLATE/Bean.java |
sed "s;@BNAME@;$BNAME;g" |
sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	

# --------------------------------------------------------
# Build Home Interface file
# --------------------------------------------------------
FILE=${BNAME}Home.java
>$FILE

cat $TEMPLATE/Home.java |
sed "s;@BNAME@;$BNAME;g" |
sed "s;@PACKAGE@;$PACKAGE;" >>$FILE

case $TYPE in
e*)
    cat $TEMPLATE/EHome.java |
    sed "s;@BNAME@;$BNAME;g" |
    sed "s;@PKCLASS@;$PKCLASS;g" >>$FILE
    ;;
esac

echo "}" >>$FILE

# --------------------------------------------------------
# Build Makefile
# --------------------------------------------------------
FILE=${BNAME}.mk
>$FILE
cat $TEMPLATE/M1 | 
sed "s;@PACKAGE@;$PACKAGE;" | 
sed "s;@BNAME@;$BNAME;g" >>$FILE

# --------------------------------------------------------
# Build a jonas.properties file
# --------------------------------------------------------
FILE=jonas.properties
>$FILE
echo "# List of deployment descriptors. " >>$FILE
echo "# In case of multiple .xml, names must be separated by comma." >>$FILE
echo "# Do not list the jonas-XXX.xml files here." >>$FILE
echo "jonas.beans.descriptors	${BNAME}.xml" >>$FILE


# --------------------------------------------------------
# Build a mini client for this bean
# --------------------------------------------------------
FILE=${BNAME}Client.java
>$FILE
cat $TEMPLATE/Client.java |
    sed "s;@BNAME@;$BNAME;g" |
    sed "s;@PACKAGE@;$PACKAGE;" >>$FILE

# --------------------------------------------------------
# ok
# --------------------------------------------------------
echo "Your bean files have been created. You can now customize them."
