// @BNAME@Bean.java
// Stateful Session Bean

package @PACKAGE@;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.RemoveException;
import javax.ejb.EJBObject;
import javax.ejb.deployment.ControlDescriptor;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.SessionSynchronization;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import javax.transaction.NotSupportedException;
import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import org.objectweb.jonas.common.Trace;


/**
 *
 */
public class @BNAME@Bean implements SessionBean, SessionSynchronization {

    static private final int trace = Trace.DB_1;
    SessionContext ejbContext;

    // ------------------------------------------------------------------
    // SessionSynchronization implementation
    // ------------------------------------------------------------------

    /** 
     * The afterBegin method notifies a session Bean instance that a new 
     * transaction has started, and that the subsequent business methods on
     * the instance will be invoked in the context of the transaction. 
     * The instance can use this method, for example, to read data from a
     * database and cache the data in the instance fields. 
     * This method executes in the proper transaction context. 
     *
     * @throws EJBException Thrown if the instance could not perform
     * the function requested by the container because of a system-level error.
     */
    public void afterBegin() {
	Trace.outln(trace, "@BNAME@Bean afterBegin");
    }


    /** 
     * The beforeCompletion method notifies a session Bean instance that a
     * transaction is about to be committed. The instance can use this method,
     * for example, to write any cached data to a database. 
     * This method executes in the proper transaction context. 
     * Note: The instance may still cause the container to rollback the transaction
     * by invoking the setRollbackOnly() method on the instance context, or by throwing
     * an exception. 
     *   
     * @throws EJBException Thrown by the method to indicate a failure caused by a
     * system-level error.
     */
    public void beforeCompletion() {
	Trace.outln(trace, "@BNAME@Bean beforeCompletion");
    }


    /** 
     * The afterCompletion method notifies a session Bean instance that a
     * transaction commit protocol has completed, and tells the instance whether
     * the transaction has been committed or rolled back. 
     * This method executes with no transaction context. 
     *
     * @param : committed - True if the transaction has been committed, false if
     * it has been rolled back.
     * 
     * @throws EJBException Thrown by the method to indicate a failure caused by a
     * system-level error.
     */
    public void afterCompletion(boolean committed) {
	Trace.outln(trace, "@BNAME@Bean afterCompletion");
    }
