/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id$
 *-----------------------------------------------------------------------------
 */





package com.lutris.classloader;

// lutris packages
import com.lutris.logging.LogChannel;

// io packages
import java.net.URL;
import java.net.URLConnection;

// io exceptions
import java.net.MalformedURLException;

// io packages
import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;

// io exceptions
import java.io.IOException;
import java.io.FileNotFoundException;

/**
 * <P>A <CODE>Resource</CODE> that is a file on a remote machine in 
 * a specified directory.  The directory is represented by a 
 * <CODE>ClassPathEntry</CODE>, and the filename is specified by a String.
 *
 * @author Kristen Pol, Lutris Technologies
 * @version $Revision : 1.1 $
 * @see com.lutris.classloader.MultiClassLoader
 * @see com.lutris.classloader.ClassPathEntry
 * @see com.lutris.classloader.Resource
 * @see java.io.File
 */
public class RemoteDirResource extends Resource {

    // data members

    private URL url = null;

    // constructors

    // FIXME: Test and change to protected and add javadoc. (kp)
    private RemoteDirResource(String name, ClassPathEntry location,
                              LogChannel loadLogChannel) 
	    throws FileNotFoundException {
	super(name, location, loadLogChannel);
	
	// Get location's URL
        URL locURL = location.getURL();
        if (locURL == null) {
            throw new FileNotFoundException( "The URL for location, "
                                             + location + ", is null");
        }

	// Create a new URL from location's URL and resource name
	try {
	    url = new URL(locURL.toString() + name);
	} catch (MalformedURLException mue) {
            throw new FileNotFoundException("The URL can not be created from the name "
                                            + name + ", and location " + locURL
                                            + ": " +mue.getMessage());
	}

	// Get the URLConnection so size and time can be determined
	URLConnection connection;
	try {
	    connection = url.openConnection();
	} catch (IOException ioe) {
            throw new FileNotFoundException("URL, " + url
                                            + ", does not exist or can not be reached");
        }
        size = connection.getContentLength();
        lastModifiedTime = connection.getLastModified();
        //FIXME: Connection must be closed
    }

    // public methods

    public InputStream getInputStream() throws IOException {
	return url.openStream();
    }

    /**
     * Get current last-modification time of resource.  This is the
     * time on the disk file the resource is associated with.
     *
     * @return the last-modified time of the permanent copy of the resource
     * in milliseconds.
     */
    public long getCurrentLastModifiedTime() {
        return -1; //FIXME: IMPLEMENT
    }
}
