/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id$
 *-----------------------------------------------------------------------------
 */





package com.lutris.logging;


/**
 * Interface of a channel associated with a logging facility.  All messages
 * for the facility are written using a channel. 
 *
 * @author Mark Diekhans
 * @see com.lutris.logging.Logger
 */
public interface LogChannel {
    
    /**
     * Determine if logging is enabled for the specified level.  This
     * is useful to prevent a series of unnecessary logging calls,
     * as often encountered with debug logging, or a call where generating
     * the message is expensive.
     *
     * @param level Numeric level that is to be checked.
     * @return <CODE>true</CODE> if enabled, <CODE>false</CODE> if not
     *         enabled.
     */
    boolean isEnabled(int level);

    /**
     * Determine if logging is enabled for the specified level.  This
     * is useful to prevent a series of unnecessary logging calls,
     * as often encountered with debug logging, or a call where generating
     * the message is expensive.
     *
     * @param level Symbolic level that is to be checked.
     * @return <CODE>true</CODE> if enabled, <CODE>false</CODE> if not
     *         enabled.
     */
    boolean isEnabled(String level);

    /**
     * Convert a symbolic level to an integer identifier.
     *
     * @param level Symbolic level that is to be checked.
     * @return The numeric level identifier
     */
    int getLevel(String level);

    /**
     * Write a string to the log file.
     *
     * @param level Numeric level the message is associated with.
     * @param msg The message to log.
     */
    void write(int level, String msg);
    
    /**
     * Write a string to the log file.
     *
     * @param level Symbolic level the message is associated with.
     * @param msg The message to log.
     */
    void write(String level, String msg);
    
    /**
     * Write a string and exception to the log file.
     *
     * @param level Numeric level the message is associated with.
     * @param msg The message to log.
     * @param throwable Exception or error to log.
     */
    void write(int level, String msg, Throwable throwable);
    
    
    /**
     * Write a string and exception to the log file.
     *
     * @param level Symbolic level the message is associated with.
     * @param msg The message to log.
     * @param throwable Exception or error to log.
     */
    void write(String level, String msg, Throwable throwable);
}
