/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id$
 *-----------------------------------------------------------------------------
 */




package com.lutris.util;

/*
 * Class that encapsulates exceptions as runtime errors.  This is used when an
 * an exception is caught that really should be treated as a runtime error.
 * This should only be used for exceptions that really should be treated as
 * fatal errors; not as a way of by-passing Java error handling.
 */
public class FatalExceptionError extends Error {
    /**
     * Encapsulated exception.
     */
    Exception except;

    /**
     * Constucts with an existing exception or error.
     *
     * @param except - Exception to encapsulate.
     */
    public FatalExceptionError (Exception except) {
        this.except = except;
    }

    /**
     * Get the exception that occured.
     *
     * @return A reference to the exception.
     */
    public Exception getException () {
        return except;
    }

    /**
     * Get the detail message of the encapsulate exception.
     *
     * @return The detail message
     */
    public String getMessage() {
	return except.getMessage();
    }

    /**
     * Get the localized message of the encapsulate exception.
     * 
     * @return The detail message
     */
    public String getLocalizedMessage() {
	return except.getLocalizedMessage();
    }

    /**
     * Returns a string representation that includes this object and the
     * encapsulated object.
     *
     * @return  The string representation.
     */
    public String toString() {
	String str = getClass().getName() + ": " +
            except.getClass().getName();
	String message = except.getMessage();

        if (message == null) 
            return str;
        return str + ": " + message;
    }

    /**
     * Call the encapsulated object printStackTrack method with output to the
     * standard error string.
     */
    public void printStackTrace() { 
        except.printStackTrace();
    }

    /**
     * Call the encapsulated object printStackTrack method with output to a
     * specified stream.
     */
    public void printStackTrace(java.io.PrintStream str) { 
        except.printStackTrace(str);
    }

    /**
     * Call the encapsulated object printStackTrack method with output to a
     * specified stream print writer.
     */
    public void printStackTrace(java.io.PrintWriter pw) {
	except.printStackTrace(pw);
    }
}
