/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.adm;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.NameClassPair;

import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;
import org.objectweb.jonas.common.RemoteObject;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.container.Container;
import org.objectweb.jonas.naming.ContainerNaming;
import org.objectweb.jonas.naming.NamingManager;
import org.objectweb.jonas.dbm.DataBaseManager;
import org.objectweb.jonas.container.EJBService;
import org.objectweb.jonas.container.EJBServiceImpl;


/**
 * This class implements a remote interface used for administering
 *      the server.
 */
public class Adm extends RemoteObject implements AdmInterface {

    static final int trace = Trace.DB_18;

    public static final String ADMNAME_SUFFIX = "_Adm";

    private EJBService ejbserv  = null;
    private DataBaseManager dbm  = null;
    private InitialContext ictx = null;

    /**
     * Adm constructor
     */
    public Adm(String jonasname) throws RemoteException, NamingException  {

	Trace.outln(trace, "Adm.Adm() ");

	String name = jonasname + ADMNAME_SUFFIX;

	// Get Service references
	ejbserv = EJBServiceImpl.getInstance();
	dbm = DataBaseManager.getInstance();

	// Get an Initial Context and
	// registers the JOnAS Server Name to JNDI
	try {
	    ictx = new InitialContext();
	    ictx.rebind(name, this);
	    Trace.outln(trace, "rebind done " + name);
	} catch (NamingException e) {
	    Trace.errln("Cannot register "+name+" : " + e);
	    throw e;
	}
    }

    // ------------------------------------------------------------------
    //  AdmInterface implementation
    // ------------------------------------------------------------------

    /**
     * configure trace messages in EJB Server
     */
    public void configureTrace(String out, String err, String level) throws RemoteException {
	Trace.out(trace, "Adm.configureTrace(String out, String err, String level) ");
	Trace.configure(out, err, level);	
    }

    /**
     * Add beans
     */
    public void addBeans(String fileName) throws RemoteException {
	Trace.out(trace, "Adm.addBeans");
	try {
	    ejbserv.addBeans(fileName);
	} catch(Exception ex) {
	    throw new RemoteException("cannot add beans in container");
	}
    }

    /**
     * Remove beans
     */
    public void removeBeans(String fileName) throws RemoteException {
	Trace.out(trace, "Adm.removeBeans");
	try {
	    ejbserv.removeBeans(fileName);
	} catch(Exception ex) {
	    throw new RemoteException("cannot remove beans in container");
	}
    }

    /**
     * List beans
     */
    public String [] listBeans() throws RemoteException {
	Trace.out(trace, "Adm.listBeans");
	try {
	    return ejbserv.listBeans();
	} catch(Exception ex) {
	    throw new RemoteException("cannot list beans in container");
	}
    }
 
    /**
     * List JNDI context
     */
    public Vector listContext() {
	Trace.out(trace, "Adm.listContext() ");
	String name = null;
	Vector ret  = new Vector();
	String prop = null;
	try {
	    NamingEnumeration ne = ictx.list("");
	    for (Enumeration e = ne; e.hasMoreElements() ;) {
		name = ((NameClassPair) e.nextElement()).getName();
		Trace.outln(trace, "    name found in context = "+ name);
		ret.addElement(name);
	    }
	} catch (Exception ex) { 
	    Trace.errln("Cannot list JNDI context : " + prop);
	    Trace.errln(ex);
	    System.exit(1);
	}
	return ret;
    }

    /**
     * List Environment
     */
    public Properties listEnv() {
	Trace.out(trace, "Adm.listEnv() ");
	return Prop.svr.getFilesEnv();
    }

    /**
     * Stop the EJB Server
     */
    public void stopServer()   throws RemoteException {
	Trace.out(trace, "Adm.stopServer() ");
	ejbserv.removeAllBeans();
	dbm.unbindDataSources();
	System.exit(0);
    }
 
    /**
     * To test if an Adm is reachable
     */
    public void ping() throws RemoteException {
    }
}
