/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.adm;


import java.io.BufferedReader; 
import java.io.InputStreamReader; 
import java.io.IOException;
import java.util.StringTokenizer;
import javax.naming.InitialContext;
import javax.naming.Context;
import java.util.Properties;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.RemoteException;

import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;



/*
 * This class implements a rmi client to administrate the server.
 * It is an interactive version of the previous JonasAdmin
 */
public class JonasAdmin { 

    static private BufferedReader in;

    Context    initialContext   = null;
    String     jonasName        = "jonas"; // default value
    String     traceOutput      = "out";   // default value
    String     traceError       = "err";   // default value

    AdmInterface admI = null;
    boolean reachable = false;
    boolean interactiveMode = false;

    public JonasAdmin(String name, boolean mode) {
	jonasName = name;
	interactiveMode = mode;

	// Gets reference on Adm object
	String admName  = jonasName + Adm.ADMNAME_SUFFIX ;
	try {
	    initialContext = new InitialContext();
	    admI = (AdmInterface) initialContext.lookup(admName);
	    admI.ping();
	    reachable = true;

	} catch (Exception e) {
	    Trace.errln ("Cannot reach  : " + admName +" "+ e);
	    System.exit(2);
	}
    }

    public static void main(String args[]) throws IOException { 

	String     beansFileName    = null;

	String     traceLevel       = null;
	boolean    lOption          = false;
	boolean    jOption          = false;
	boolean    eOption          = false;
	boolean    aOption          = false;
	boolean    tOption          = false;
	boolean    hOption          = false;
	boolean    sOption          = false;
	boolean    iOption          = true;

	JonasAdmin admt = null;

	Trace.configure();
	String  currentName = "jonas";
	// Command line
	String commandLine = "JonasAdmin";
	for (int argn=0; argn<args.length; argn++) {
	    String arg = args[argn];
	    
	    if ((argn<args.length) && (arg.equals("-l") == true)) {
		commandLine = commandLine + " -l ";
		lOption = true;
		iOption = false;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-s") == true)) {
		commandLine = commandLine + " -s ";
		sOption = true;
		iOption = false;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-a") == true)) {
		beansFileName = args[++argn];
		commandLine = commandLine + " -a "+beansFileName;
		aOption = true;
		iOption = false;
		continue;
	    }
	    if ((argn<args.length) && (arg.equals("-n") == true)) {
		currentName = args[++argn];
		continue;
	    }
	    if ((argn<args.length) && (arg.equals("-i") == true)) {
		commandLine = commandLine + " -i ";
		iOption = true;
		continue;
	    }
	    if ((argn<args.length) && (arg.equals("-t") == true)) {
		traceLevel = args[++argn];
		commandLine = commandLine + " -t "+traceLevel;
		tOption = true;
		iOption = false;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-j") == true)) {
		commandLine = commandLine + " -j ";
		jOption = true;
		iOption = false;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-e") == true)) {
		commandLine = commandLine + " -e ";
		eOption = true;
		iOption = false;
		continue;
	    }


	    if ((argn<args.length) && ((arg.equals("-h") == true) || 
				       (arg.equals("-?") == true))) {
		commandLine = commandLine + " -?";
		hOption = true;
		usage();
		System.exit(0);
	    }
	}
	

	admt = new JonasAdmin(currentName, iOption); 

	if (iOption) {
	    admt.menu();
	}else { 
	    if (aOption) {
		// Adds beans
		admt.addBeans(beansFileName);	
	    }
	    if (lOption) {
		// Lists beans
		admt.listbeans();
	    }
	    if (jOption) {
		// jndi names
		admt.listnames();
	    }
	    if (eOption) {
		// Lists JOnAS properties 
		admt.listproperties();
	    }
	    if (tOption) {
		// set JOnAS trace level
		admt.modTrace(traceLevel);
	    }
	    if(sOption){
		// Stops Server
		admt.stopserver();
	    }
		
	}
    }

    public void prompt() {
	System.out.print("JonasAdmin ("+jonasName+") > ");
    }

    public void menu() throws IOException {
	System.out.println("Current EJBServer : " + jonasName);

	// User interface
	in = new BufferedReader(new InputStreamReader(System.in));
	String  command;
	char c ;

	prompt();;
	while ( (command = in.readLine()).length() != 0) {
	    switch (c = command.charAt(0)) {
	    case 'q' :
		System.exit(2);

	    case 's' :
		stopserver();
		break;
	    case  'l' :
		listbeans();
		break;
	    case  'j' :
		listnames();
		break;
	    case  'e' :
		listproperties();
		break;
	    case  'a' :
		String     bFName    = null;
		System.out.print("file name ? > ");
		if( (bFName = in.readLine()).length() != 0) { 
		    addBeans(bFName);
		}
		break;
	    case  't' :
		String  level = null;
		System.out.print("trace level ? > ");
		if( (level = in.readLine()).length() != 0) { 
		    modTrace(level);
		}
		break;
	    case  'n' :
		changeserver();
		break;
	    case  'h' :	
		help();
		break;
	    default :
		System.out.println("unknown command ");
		help();
		break;
	    }
	    prompt();
	}
    }
	    
    private void addBeans(String beansFileName) {

	if (!reachable) {
	    System.out.println("EJBServer "+jonasName+" unreachable");
	    return;
	}

	// Properties from file
	Prop fileProp  = new Prop(beansFileName);
	String descNames = fileProp.getValue("jonas.beans.descriptors");
	// Loops on beans descriptors names
	try {
	    StringTokenizer st = new StringTokenizer(descNames, ",");
	    while(st.hasMoreTokens()) {
		admI.addBeans(st.nextToken());
	    }
	} catch (RemoteException e) {
	    Trace.errln("admI.addBeans("+descNames+") : "+e);
	    System.exit(2);
	}
    }

    private void listbeans() {

	if (!reachable) {
	    System.out.println("EJBServer "+jonasName+" unreachable");
	    return;
	}

	try {
	    String[] result = admI.listBeans();
	    if (result.length == 0)
		System.out.println("No bean in JOnAS server.");
	    for (int i = 0; i < result.length; i++) {
		System.out.println(result[i]);
	    } 
	} catch (RemoteException e) {
	    Trace.errln ("admI.listBeans() : "+e);
	    System.exit(2);
	}
    }

    private void listnames() {

	if (!reachable) {
	    System.out.println("EJBServer "+jonasName+" unreachable");
	    return;
	}

	Vector result = new Vector();
	try {
	    result = admI.listContext();
	} catch (RemoteException ex) {
	    Trace.errln ("admI.listBeans() : "+ex);
	    System.exit(2);
	}
	    
	if (result.size() == 0)
	    System.out.println("No name in JNDI context.");
	for (int i = 0; i < result.size(); i++) {
	    System.out.println(result.elementAt(i));
	}
    }

    private void listproperties() {

	if (!reachable) {
	    System.out.println("EJBServer "+jonasName+" unreachable");
	    return;
	}

	Properties p = new Properties();
	try {
	    p = admI.listEnv();
	} catch (RemoteException ex) {
	    Trace.errln ("admI.listBeans() : "+ex);
	    System.exit(2);
	}
	    
	for (Enumeration e = p.keys(); e.hasMoreElements();) {
	    Object key   = e.nextElement();
	    Object value = p.get(key);
	    System.out.println(key.toString() + "=" + value.toString());
	}	

    }

    private void modTrace(String level) {
	if (reachable){
	    try {
		admI.configureTrace (traceOutput, traceError, level);
	    } catch (RemoteException e) {
		Trace.errln ("admI.configureTrace : "+e);
	    }
	    System.out.println("Server traces set to "+ level);
	}
	
    }

    private void changeserver() {
	String name = null;
	System.out.print ("Enter the EJBServer's name (jonas.name property) : ");
	try {
	    name = new String(in.readLine());
	}catch (IOException ex){
	    System.out.println("cannot readLine " +ex);
	    return;
	}

	if((name != null) || (name == "")){
	    String admName  = name + Adm.ADMNAME_SUFFIX ;
	    try {
		initialContext = new InitialContext();
		admI = (AdmInterface) initialContext.lookup(admName);
		admI.ping();
		jonasName = name;
		reachable = true;
	    } catch (Exception e) {
		Trace.errln ("Cannot reach  : " + admName +" "+ e);
		return;
	    }
	    System.out.println("The Current EJBServer is now : "+name);
	}else{
	   System.out.println("Incorrect value for EJBServer name");
	} 
    }

    private void stopserver() {
	if (reachable){
	    try{
		admI.stopServer();
		System.out.println("EJBServer "+jonasName+ " stopped");
		reachable = false;
	    }catch (RemoteException ex) {
		System.exit(0);
	    }
	}else 
	    System.out.println("EJBServer "+jonasName+" unreachable");  	
    }

    private void help() {
	System.out.println("quit         quit JonasAdmin");
	System.out.println("stop         stop current EJBServer");
	System.out.println("list beans   lists types of beans");
	System.out.println("jndi names   lists registered JNDI names");
	System.out.println("env          JOnAS properties used by the server");
	System.out.println("add beans    adds types of beans in the server");
	System.out.println("trace        set traces in server");
	System.out.println("name         to identify a current EJBServer");
	System.out.println("help         help");
    }

    static void usage() {
	System.out.println (" usage : JonasAdmin [ -n JonasName ]");
	System.out.println ("                  [ -i ]" );
	System.out.println ("                  [ -s ] [ -l ] [ -j ] [ -E ]");  
	System.out.println ("                  [ -a  BeansFileName ] ");
	System.out.println ("                  [ -t {<flag1>,<flag2>,<flag3>,...} ]");
	System.out.println ("                  [ -h | -? ]" );
	System.out.println ("" );
	System.out.println ("    -n JonasName : to identify an EJBServer");
	System.out.println ("    -i : Interactive mode");
	System.out.println ("    -s : stops the EJB server.");
	System.out.println ("    -l : lists types of beans currently in the EJB Server.");
	System.out.println ("    -j : lists registered JNDI names.");
	System.out.println ("    -e : lists JOnAS properties currently used by the EJB Server.");
	System.out.println ("    -a BeansFileName : dynamically adds types of beans in the EJB Server.");

	System.out.println ("    -h : prints this message.");
	System.out.println ("    -? : prints this message.");
    }
}
