/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * 00/13/03 Vincent Sheffer
 *          Exit properly on failure of the external command.
 * 00/02/05 Nozomi Matsumura
 *          GenIC i18n problem.
 *          GenIC invoke javac and javac's output has non asc-ii characters,
 *          javac's output did not show collectly.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.common;


/**
 * This class allows to run a command in sub-process.
 */

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintStream;
import java.lang.Process;
import java.lang.String;


public class Cmd {

    private String mCmd = null;

    /**
     * Construtor method of Cmd class.
     * @param cmd command
     */
    public Cmd(String cmd) {
	mCmd = cmd;
    }

    /**
     * Execute the command.
     * (In case of the exit value of the command is not 0,
     * the output and error streams of the command is traced.)
     * @return false if the command cannot be executed, or if its exit value is not 0.
     */
    public boolean run() {
	RunnableStreamListener out, err;
	Process proc = null;
	boolean val = true;

	try {
	    proc = Runtime.getRuntime().exec(mCmd);
	} catch (IOException e) {
	    Trace.errln(e, true);
	    return(false);
	}
	out = new RunnableStreamListener(new BufferedReader(new InputStreamReader(proc.getInputStream())),
					 System.out);
	new Thread(out, "stdout listener for "+mCmd).start();
	err = new RunnableStreamListener(new BufferedReader(new InputStreamReader(proc.getErrorStream())),
					 System.err);
	new Thread(err, "stderr listener for "+mCmd).start();
	try {
	    val = proc.waitFor() == 0;
	} catch (InterruptedException e) {
	    Trace.errln(e, true);
	    val = false;
	}

	return(val);
    }
}


//  this runs happily, swallowing the stdout from the process
class RunnableStreamListener implements Runnable {
    BufferedReader in_;
    PrintStream stream_;
    RunnableStreamListener(BufferedReader in, PrintStream stream ) {
        in_ = in;
        stream_ = stream;
    }
    public void run() {
        String line;
        try {
            while ((line = in_.readLine()) != null)
            stream_.println(line);
        } catch (IOException e ) {
            stream_.println(e.toString());
        }
    }
}

