/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.common;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.String;


/**
 * The IndentedFile Class allows to create an ASCCI file with indented lines.
 * This class is used for the creation of the implementation classes sources
 * of the Bean's interfaces via GenIC.
 */

public class IndentedFile {

    private int         curIndentation;
    private PrintWriter fOut;
    private String      fName;

    /**
     * Cerates a new IndentedFile with the given name.
     *  @param name is the name of the IndentedFile to create
     */
    public IndentedFile(String name) throws IOException {
	curIndentation = 0;
	fName = new String(name);
	int li = fName.lastIndexOf(File.separatorChar);
	if (li != -1) {
	    String dirName = fName.substring(0, li);
	    File dir = new File(dirName);
	    if (!dir.exists()) {
		if (!dir.mkdirs()) {
		    throw new IOException("Cannot create the directory '"+dirName+"'");
		}
	    }
	}
	fOut = new PrintWriter(new FileWriter(name));
    }

    /**
     * Increment the indentation value.
     */
    public void indentPlus() {
	curIndentation++;
    }

    /**
     * Decrement the indentation value.
     */
    public void indentMinus() {
	curIndentation--;
    }

    /**
     * Print the given message with or without indentation.
     */
    public void print(String msg, boolean isIndent) {
	if (isIndent == true) {
	    String blank = new String();
	    for (int i=0; i < curIndentation; i++) {
		blank = blank.concat("    ");
	    }
	    fOut.print(blank);
	}
	fOut.print(msg);
	fOut.flush();
    }

    /**
     * Print the given message with indentation.
     */
    public void print(String msg) {
	print(msg, true);
    }

    /**
     * Finish the line.
     */
    public void println() {
	fOut.println();
    }

    /**
     * Print the given message with indentation, and then finish the line.
     */
    public void println(String msg) {
	print(msg, true);
	fOut.println();
    }

    /**
     * Print the given message with or without indentation, and then finish the line.
     */
    public void println(String msg, boolean isIndent) {
	print(msg, isIndent);
	fOut.println();
    }

    /**
     * Close the file
     */
    public void close() {
	fOut.close();
    }

    /**
     * Close and delete the file
     */
    public void delete() {
	close();
	File f = new File(fName);
	f.delete();
    }
}
