/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.container; 

import javax.transaction.Status;
import javax.transaction.Synchronization;
import org.objectweb.jonas.common.Trace;


/**
 * This class is used in JEntityHome class in putEJBObject
 * in order to have a transactional behavior on create 
 * the ejbobjects will be update if an EJB Object had been created during
 * a rolled back transaction (cf bug 98.11.30.1)
 *
 * XXX PhD XXX
 * Unfortunatly, this doesn't work if setRollbackOnly is done in ejbPostCreate
 * because the registerSynchronization is done too late (rolledback exception!!!)
 */
public class HomeSynchro implements Synchronization {

    private static final int trace = Trace.DB_17;

    protected JEntityHome home = null;
    protected Object primaryKey = null;

    /**
     */
    public HomeSynchro(JEntityHome h, Object pk) {
	home = h;
	primaryKey = pk;
    }
    
   /** 
     * 
     */
    public void beforeCompletion() {
	Trace.outln(trace, "HomeSynchro.beforeCompletion()");
    }


    /** 
     * 
     */
    public void afterCompletion(int status) {
	Trace.outln(trace, "HomeSynchro.afterCompletion(Status "+status+") ");

	if (status != Status.STATUS_COMMITTED) {
	    Trace.outln(trace, "HomeSynchro.afterCompletion :home.removeEJBObject(primaryKey)");
	    home.removeEJBObject(primaryKey);
	}
    }
}
