/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import javax.ejb.EJBHome;
import javax.ejb.EJBMetaData;
import java.io.Serializable;
import org.objectweb.jonas.common.Trace;

/**
 * This class implements the javax.ejb.EJBMetaData interface.
 *
 */
public class JBeanMetaData implements EJBMetaData, Serializable {

    private static int trace = Trace.DB_16;

    /**
     * @serial
     */
    private EJBHome home;

    /**
     * @serial
     */
    private String   homeInterfaceClass;

    /**
     * @serial
     */
    private String   remoteInterfaceClass;

    /**
     * @serial
     */
    private String   primaryKeyClass;

    /**
     * @serial
     */
    private boolean isSession;

    /**
     * @serial
     */
    private boolean isStatelessSession;

    public JBeanMetaData(JBeanHome ejbHome,
			    String  homeInterfaceClassName,
			    String  remoteInterfaceClassName,
			    String  primaryKeyClassName,
			    boolean isSession,
			    boolean isStatelessSession) {
	
	Trace.outln(trace, "JBeanMetaData.JBeanMetaData(ejbHome," 
		    +homeInterfaceClassName+", "+remoteInterfaceClassName
		    +", "+primaryKeyClassName+", "+isSession+", "+isStatelessSession+")");
	this.home = ejbHome;
	this.homeInterfaceClass   = homeInterfaceClassName;
	this.remoteInterfaceClass = remoteInterfaceClassName;
	this.primaryKeyClass      = primaryKeyClassName;
	this.isSession = isSession;
	this.isStatelessSession = isStatelessSession;
    }


    /**
     * Obtains the home interface of the enterprise Bean. 
     */
    
    public EJBHome getEJBHome() {
	Trace.outln(trace, "JBeanMetaData.getEJBHome() ");
	Trace.errln("EJBMetaData.getEJBHome(): Not yet implemented");
	return home;
    }
    


    /**
     * Obtains the Class object for the enterprise Bean's home interface. 
     */
    public Class getHomeInterfaceClass() {
	Trace.outln(trace, "JBeanMetaData.getHomeInterface() ");
	Class c = null;
	try {
	    c = Class.forName(homeInterfaceClass);
	} catch(java.lang.ClassNotFoundException e) {
	    Trace.errln("Class.forName() : " + e);
	}
	return c;
    }


    /**
     * Obtains the Class object for the enterprise Bean's primary key class. 
     */
    public Class getPrimaryKeyClass() {
	Trace.outln(trace, "JBeanMetaData.getPrimaryKeyClass() ");
	if(isSession)
	    throw new RuntimeException("No PrimaryKey Class for session Bean");
	Class c = null;
	try {
	    c = Class.forName(primaryKeyClass);
	} catch(java.lang.ClassNotFoundException e) {
	    Trace.errln("Class.forName() : " + e);
	}
	return c;
    }


    /**
     * Obtains the Class object for the enterprise Bean's remote interface. 
     */
    public Class getRemoteInterfaceClass() {
	Trace.outln(trace, "JBeanMetaData.getRemoteInterfaceClass() ");
	Class c = null;
	try {
	    c = Class.forName(remoteInterfaceClass);
	} catch(java.lang.ClassNotFoundException e) {
	    Trace.errln("Class.forName() : " + e);
	}
	return c;
    }
    

    /**
     * Tests if the enterprise Bean's type is "session".
     */
    public boolean isSession() {
	Trace.outln(trace, "JBeanMetaData.isSession() = " + isSession);
	return isSession;
    }

    /**
     * Test if the enterprise Bean's type is "stateless session".
     * SINCE EJB 1.1
     *
     * @return True if the type of the enterprise Bean is stateless session.
     */
    public boolean isStatelessSession() {
	Trace.outln(trace, "JBeanMetaData.isStatelessSession() = " + isStatelessSession);
	return isStatelessSession;
    }
}


 
