/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */



package org.objectweb.jonas.container; 

import java.rmi.RemoteException;
import javax.ejb.RemoveException;
import javax.ejb.Handle;
import javax.ejb.EJBObject;
import javax.transaction.Transaction;

import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.SessionDesc;
import org.objectweb.jonas.deployment.api.SessionStatefulDesc;
import org.objectweb.jonas.jtm.Current;

/**
 *
 */
public class JSessionHome extends JBeanHome {

    private final static int trace = Trace.DB_16;

    /**
     * @serial
     */
    private boolean isStateFul = false;

    /**
     * constructor
     */
    public JSessionHome(SessionDesc desc, String beanName) throws RemoteException {

	super(desc, beanName);

	isStateFul = (desc instanceof SessionStatefulDesc);

	TxBeanManaged = (desc.getTransactionType() == SessionDesc.BEAN_TRANSACTION_TYPE);

	JBeanMetaData bbmd = new JBeanMetaData(this, 
					       desc.getHomeClass().getName(),
					       desc.getRemoteClass().getName(),
					       null,
					       true,
					       !isStateFul);
	this.setEJBMetaData(bbmd);
    }
    
    public boolean isStateFul() {
	return isStateFul;
    }

    /**
     * Removes an EJB object identified by its handle. 
     */
    public void remove(Handle handle) throws RemoteException, RemoveException{
	Trace.outln(trace,"remove(handle)");
	EJBObject ejbobj = handle.getEJBObject();
	ejbobj.remove();
    }

    /**
     * Removes an EJB object identified by its primary key.
     */

    public void remove(Object primaryKey) throws RemoteException, RemoveException {
	throw new RemoveException("There is no primary key on a sessionBean");
    }

    /**
     * consider transaction for create and finder methods
     * nothing to do for session beans
     */
    protected RequestCtx preinvoke(int txattr) {

	Trace.outln(trace,"JSessionHome.preinvoke");

	// Don't check transaction attribute for session beans
	boolean commit = false;
	Transaction cltx = null;

	// Build a RequestCtx to save information about this request
	// until the postinvoke time.
	RequestCtx rctx = new RequestCtx(cltx, commit);

	// Set bean context for JNDI
	rctx.setOldJNDICtx(setComponentContext());

	// Set the connectionList in threadLocal
	// and save old value in RequestContext
	rctx.setOldConnectionList(Current.setConnectionList(connectionList));

	return rctx;
    }

}
