/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import java.util.Vector;
import javax.ejb.EJBContext;
import javax.naming.Context;
import javax.transaction.Transaction;
import org.objectweb.jonas.common.Trace;

/**
 * Context associated to a request to a bean.
 * mandatory for finder methods that return collections.
 * used for all business methods.
 */
public class RequestCtx { 

    private static final int trace = Trace.DB_12;

    private Transaction clientTransaction = null; // Tx to be resumed at postinvoke
    private boolean mustBeCommitted = false; // must commit current Tx at postinvoke
    private Throwable sysExc = null; // system exception or error raised in business method
    private Context oldJNDIctx = null; // JNDICtx to be resumed at postinvoke
    private Vector oldConnectionList = null;
    private EJBContext ejbContext = null; // used for regular business methods only
    private int txattr = 0;	// used for regular business methods on session beans

    /**
     * constructor. Called at preinvoke.
     */
    public RequestCtx(Transaction cltx, boolean commit) {
	clientTransaction = cltx;
	mustBeCommitted = commit;
    }

    /**
     * Obtains the client Transaction to be resumed at postinvoke
     */
    public Transaction getClientTx() {
	return clientTransaction;
    }

    /**
     * return true if must commit the current transaction at postinvoke
     */
    public boolean mustCommit() {
	return mustBeCommitted;
    }    

    /**
     * set system exception in context
     */
    public void setSysExc(Throwable e) {
	this.sysExc = e;
    }

    /**
     * get system exception in context
     */
    public Throwable getSysExc() {
	return this.sysExc;
    }

    /**
     * set JNDICtx
     */
    public void setOldJNDICtx(Context c) {
	this.oldJNDIctx = c;
    }

    /**
     * get JNDICtx
     */
    public Context getOldJNDICtx() {
	return this.oldJNDIctx;
    }

    /**
     * save Connection List
     */
    public void setOldConnectionList(Vector list) {
	this.oldConnectionList = list;
    }

    /**
     * get saved Connection List
     */
    public Vector getOldConnectionList() {
	return this.oldConnectionList;
    }

    /**
     * set ejbContext
     */
    public void setEJBContext(EJBContext c) {
	this.ejbContext = c;
    }

    /**
     * get ejbContext
     */
    public EJBContext getEJBContext() {
	return this.ejbContext;
    }
   
    /**
     * set TxAttr
     */
    public void setTxAttr(int a) {
	this.txattr = a;
    }

    /**
     * get TxAttr
     */
    public int getTxAttr() {
	return this.txattr;
    }
   
}
