/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2000 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.dbm;

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;
import org.objectweb.jonas.common.Trace;


public class DataSourceFactory implements ObjectFactory {

    private static final int trace = Trace.DB_8;

    public Object getObjectInstance(Object refObj, Name name, Context nameCtx, Hashtable env) throws Exception {

	Reference ref = (Reference) refObj;

	if (ref.getClassName().equals("org.objectweb.jonas.dbm.ConnectionManager")) {
	    String dsname = (String)ref.get("datasource.name").getContent();
	    ConnectionManager ds = ConnectionManager.getConnectionManager(dsname);
	    if (ds == null) {
		// Should never go here, because DataSource is already created in Container!
		Trace.errln("Creating a new Connection Manager for "+dsname);
		ds = new ConnectionManager();
		ds.setDataSourceName(dsname);
		ds.setUrl((String)ref.get("datasource.url").getContent());
		ds.setClassName((String)ref.get("datasource.classname").getContent());
		ds.setUserName((String)ref.get("datasource.username").getContent());
		ds.setPassword((String)ref.get("datasource.password").getContent());
	    } else {
		Trace.outln(trace, "DataSourceFactory: return the matching ConnectionManager");
	    }
	    return ds;
	} else {
	    Trace.errln("DataSourceFactory.getObjectInstance(): No object found");
	    return null;
	}
    }
}
