/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2000 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.dbm;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import javax.sql.XAConnection;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.jtm.TransactionImpl;

/**
 * This class represents a connection stored in the pool.
 * It may be in different states:
 * - available and reuseable for this user (open=0, tx=null)
 * - reusable for the same transaction, not already committed (open=?, tx!=null)
 * - still open, but not involved in a Tx (open=1, tx=null)
 * - busy (open>jdbcConnMaxThreads, tx!=null)
 * These 2 cases must be considered: close after commit, commit after close.
 * We cannot suppose what shema the user will use and we must handle both correctly.
 */
public class PoolItem {
    private static int count = 0;
    public XAConnection xaConn; // the pooled connection
    public String user;         // the user name
    public int open;            // >0 if connection is open
    public Transaction tx;  // transaction the connection is involved with
    public long lMaxAgeMillis;  // When it is expected to die
    public int number;
    
    /*
     * constructor used by newConnection()
     */
    public PoolItem(XAConnection xac, String user, long maxage) {
        this.xaConn = xac;
        this.user = user;
        open = 0;
        lMaxAgeMillis = System.currentTimeMillis() + maxage;
	number = count++;
    }

    /*
     * string form for debugging
     */
    public String toString() {
	Integer i = new Integer(number);
	return i.toString();
    }
}
