/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A. 
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Lutris Technologies Inc http://www.lutris.com
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
* several changes 13.3.00
*/


package org.objectweb.jonas.deployment.api;

import java.lang.Class;
import java.lang.String;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;

import org.objectweb.jonas.deployment.xml.ContainerTransaction;
import org.objectweb.jonas.deployment.xml.Method;
import org.objectweb.jonas.deployment.xml.DisplayName;
import org.objectweb.jonas.deployment.xml.EjbName;
import org.objectweb.jonas.deployment.xml.Home;
import org.objectweb.jonas.deployment.xml.Remote;
import org.objectweb.jonas.deployment.xml.EjbClass;
import org.objectweb.jonas.deployment.xml.EnvEntry;
import org.objectweb.jonas.deployment.xml.ResourceRef;
import org.objectweb.jonas.deployment.xml.EjbRef;
import org.objectweb.jonas.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas.deployment.xml.JndiName;
import org.objectweb.jonas.deployment.xml.TransAttribute;
import org.objectweb.jonas.deployment.xml.JonasResource;


/**
 * Base class for bean descriptors
 * @autor Helene Joanin, Christophe Ney
*/
public abstract class BeanDesc {

    private static final String ejb10PropertiesPrefix = "ejb10-properties/";

    protected String displayName = null;
    protected String ejbName = null;
    protected String jndiName = null;
    protected Class home = null;
    protected Class remote = null;
    protected Class ejbClass = null;
    protected Vector envEntryDesc = new Vector();
    protected Properties ejb10EnvProps = new Properties();
    protected Vector resourceRefDesc = new Vector();
    protected Vector ejbRefDesc = new Vector();
    
    // Hashtable for methods 
    protected Hashtable methodDesc = new Hashtable();
        
    /**
     * Package protected constructor used by API
     */
    BeanDesc(ClassLoader classLoader,DisplayName displayName, EjbName ejbName, 
        JndiName jndiName,Home home, Remote remote, EjbClass ejbClass,
        EnvEntry[] env,ResourceRef[] res, JonasResource[] jRes, EjbRef ref[], 
        AssemblyDescriptor asd) 
            throws DeploymentDescException {
        // names
        this.displayName = (displayName!=null)?displayName.getContent():null;
        this.ejbName = ejbName.getContent();
        this.jndiName = jndiName.getContent();
        
        // classes
        try {
            this.home = classLoader.loadClass(home.getContent());
        } catch(ClassNotFoundException e) {
            throw new DeploymentDescException("Home class not found for bean "+this.ejbName,e);
        }
        
        try {
            this.remote = classLoader.loadClass(remote.getContent());
        } catch(ClassNotFoundException e) {
            throw new DeploymentDescException("Remote class not found for bean "+this.ejbName,e);
        }
        
        try {
            this.ejbClass = classLoader.loadClass(ejbClass.getContent());
        } catch(ClassNotFoundException e) {
            throw new DeploymentDescException("Ejb class not found for bean "+this.ejbName,e);
        }
        
        // env-entries and ejb10 environment properties
        for (int i=0; i<env.length; i++) {
            envEntryDesc.addElement(new EnvEntryDesc(env[i]));
        }
    	for (Enumeration e = envEntryDesc.elements(); e.hasMoreElements(); ) {
	        EnvEntryDesc ed = ((EnvEntryDesc)e.nextElement());
	        if ( ed.getName().startsWith(ejb10PropertiesPrefix) ) {
		        String newName = ed.getName().substring(ejb10PropertiesPrefix.length());
		        String value = new String();
		        if (ed.hasValue()) {
		            value = ed.getValue().toString();
    		    }
	    	    ejb10EnvProps.put(newName, value);
	        }
	    }

    	// resources
        Hashtable resource = new Hashtable();
        for (int i=0; i<jRes.length;i++) {
            resource.put(jRes[i].getResRefName().getContent(),jRes[i]);
        }
        for (int i=0; i<res.length; i++) {
            String rName = res[i].getResRefName().getContent();
            if (!resource.containsKey(rName))
                throw new DeploymentDescException("jonas-resource missing for res-ref-name "+rName);
            resourceRefDesc.addElement(new ResourceRefDesc(classLoader,res[i],(JonasResource)resource.get(rName)));
        }
        
        // ejb-ref
        for (int i=0; i<ref.length; i++) {
            ejbRefDesc.addElement(new EjbRefDesc(ref[i]));
        }

        // populate hashtable with all methods
        try {
            java.lang.reflect.Method m[] = this.home.getMethods();
            for (int i=0; i<m.length;i++){
                if (!m[i].getDeclaringClass().equals(javax.ejb.EJBHome.class)) {
                    methodDesc.put(m[i],new MethodDesc());
                }
            }
            m = this.remote.getMethods();
            for (int i=0; i<m.length;i++){
                if (!m[i].getDeclaringClass().equals(javax.ejb.EJBObject.class)) {
                    methodDesc.put(m[i],new MethodDesc());
                }
            }
        } catch (SecurityException e){
            throw new DeploymentDescException("Cannot use java reflexion on "+this.home.getName());
        }

        // set method descriptor values for container transactions
        if (asd!=null) {
            ContainerTransaction[] ct = asd.getContainerTransaction();
            for (int i=0; i<ct.length; i++) {
                for (int j=0; j<ct[i].getMethodCount(); j++) {
                    Method m = ct[i].getMethod(j);
                    // check method applies to current bean
                    if (!m.getEjbName().getContent().equals(this.ejbName))
                        continue;
                    // selection of interface home/remote
                    Class patternExcludeClass = null;
                    if (m.getMethodIntf()!=null) {
                        if (m.getMethodIntf().getContent().equals("Home"))
                            patternExcludeClass = this.remote;
                        else if (m.getMethodIntf().getContent().equals("Remote"))
                            patternExcludeClass = this.home;
                        else
                            throw new DeploymentDescException(m.getMethodIntf().getContent()+" is invalid value for method-intf on bean "+this.ejbName);
                    }
                    // loop on methods in the hashtable
                    Enumeration e = methodDesc.keys();
                    while (e.hasMoreElements()) {
                        java.lang.reflect.Method meth = (java.lang.reflect.Method)e.nextElement();
                        // status of method name pattern matching 
                        int matchStatus = MethodDesc.matchPattern(meth,patternExcludeClass,m.getMethodName(),m.getMethodParams());                        
                        MethodDesc md = (MethodDesc)methodDesc.get(meth);
                        // set if proper level of assignation
                        md.overwriteTxAttribute(ct[i].getTransAttribute(),matchStatus);
                    }
                }
            }
        }
    }
    
                                 
    /**
     * Check that the bean descriptor is valid
     * @exception DeploymentDescException thrown for non-valid bean
     */
    public void check() throws DeploymentDescException {
        // check method descriptors
        Enumeration e = methodDesc.keys();
        while (e.hasMoreElements()) {
            java.lang.reflect.Method m = (java.lang.reflect.Method)e.nextElement();
            checkTxAttribute(m);
        }
    }
    
    /**
     * check that trans-attribute is valid for bean
     */
    protected abstract void checkTxAttribute(java.lang.reflect.Method method) throws DeploymentDescException;
    
    
    /**
     * Get the displayName of the bean.
     * Used by Server
     * @return null if not set
     */
    public String getDisplayName(){
        return displayName;
    }

    /**
     * Get the enterprise bean's name.
     * @return Name of the bean
     */
    public String getEjbName(){
        return ejbName;
    }

    /**
     * Get the name to associate with the enterprise Bean in the JNDI name space.
     * (This information is JOnAS specific). Used by Server
     * @return Jndi name of the bean
     */
    public String getJndiName(){
        return jndiName;
    }

    /**
     * Get bean's home interface.
     * Used by GenIC and Server
     * @return Class for the home interface
     */
    public Class getHomeClass(){
        return home;
    }

    /**
     * Get bean's remote interface.
     * Used by GenIC and Server
     * @return Class for the remote interface
     */
    public Class getRemoteClass(){
        return remote;
    }

    /**
     * Get bean's class.
     * Used by GenIC and Server
     * @return Class for the bean
     */
    public Class getEjbClass(){
        return ejbClass;
    }

    /**
     * Get bean's environment entries.
     * Used by Server
     * @return array of resource reference descriptors (may be empty)
     */
    public EnvEntryDesc[] getEnvEntryDesc(){
        EnvEntryDesc ret[] = new EnvEntryDesc[envEntryDesc.size()];
        envEntryDesc.copyInto(ret);
        return ret;
    }

    /**
     * Get bean's EJB 1.0 environment properties
     */
    public Properties getEjb10Environment() {
    	Properties ret = new Properties(ejb10EnvProps);
	    return ret;
    }
    
    /**
     * Get bean's resource manager connection factory references.
     * Used by Server
     * @return array of resource reference descriptors (may be empty)
     */
    public ResourceRefDesc[] getResourceRefDesc(){
        ResourceRefDesc ret[] = new ResourceRefDesc[resourceRefDesc.size()];
        resourceRefDesc.copyInto(ret);
        return ret;
    }
    
    /**
     * Get bean's ejb references.
     * Used by Server
     * @return array of ejb reference descriptors (may be empty)
     */
    public EjbRefDesc[] getEjbRefDesc(){
        EjbRefDesc ret[] = new EjbRefDesc[ejbRefDesc.size()];
        ejbRefDesc.copyInto(ret);
        return ret;
    }
    
    /**
     * Get descriptor attached to a given method.
     * Used by GenIC
     * @param method Method from the Home/Remote interface
     * @return The method descriptor (never returns null)
     */
    public MethodDesc getMethodDesc(java.lang.reflect.Method method){
        if (!methodDesc.containsKey(method))
            throw new Error(method.getName()+" is not a method of home/remote insterface for bean"+getEjbName());
        return (MethodDesc)methodDesc.get(method);
    }

    
    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\ngetDisplayName()="+getDisplayName());
        ret.append("\ngetEjbName()="+getEjbName());
        ret.append("\ngetJndiName()="+getJndiName());
        ret.append("\ngetHomeClass()="+getHomeClass().toString());
        ret.append("\ngetRemoteClass()="+getRemoteClass().toString());
        ret.append("\ngetEjbClass()="+getEjbClass().toString());
        EnvEntryDesc[] envE = getEnvEntryDesc();
        for (int i=0; i<envE.length; i++) {
            ret.append("\ngetEnvEntryDesc("+i+")="+envE[i].toString());
        }
        ResourceRefDesc[] resR = getResourceRefDesc();
        for (int i=0; i<resR.length; i++) {
            ret.append("\ngetResourceRefDesc("+i+")="+resR[i].toString());
        }
        EjbRefDesc[] ejbR = getEjbRefDesc();
        for (int i=0; i<ejbR.length; i++) {
            ret.append("\ngetEjbRefDesc("+i+")="+ejbR[i].toString());
        }
        Enumeration e = methodDesc.keys();
        while (e.hasMoreElements()) {
            java.lang.reflect.Method m = (java.lang.reflect.Method)e.nextElement();
            MethodDesc md = getMethodDesc(m);
            ret.append("\ngetMethodDesc("+m.toString()+")="+md.getClass().getName());
            ret.append(md.toString());
        }
        return ret.toString();
    }
}

