/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A. 
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Lutris Technologies Inc http://www.lutris.com
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
*/


package org.objectweb.jonas.deployment.api;

import java.io.*;

/**
 * The DeploymentDesc exception uses the pattern defined by the Enhydra 
 * com.lutris.util.ChainedException 
 * The source has been temporary duplicated to prevent Jonas using 
 * from external dependencies.
 * @author Christophe Ney
 */
public class DeploymentDescException extends Exception {

    private Throwable cause;

    /**
     * Construct an exception without a specified cause.
     *
     * @param msg The message associated with the exception.
     */
    public DeploymentDescException(String msg) {
        super(msg);
        cause = null;
    }

    /**
     * Construct an exception with an associated causing exception.
     *
     * @param msg The message associated with the exception.
     * @param cause The error or exception that cause this
     *  exception.
     */
    public DeploymentDescException(String msg,
                            Throwable cause) {
        super(msg);
        this.cause = cause;
    }

    /**
     * Construct an exception from a causing exception.
     *
     * @param cause The error or exception that cause this
     *  exception. The message will be take be this object's
     *  messasge.
     */
    public DeploymentDescException(Throwable cause) {
        super(cause.getMessage());
        this.cause = cause;
    }

    /**
     * Return the message associated with this exception.  If causes
     * are included, they will be appended to the message.
     */
    public String getMessage() {
        String msg = super.getMessage();
        if (cause == null) {
            return msg;
        } else {
            return msg + ": " + cause.getMessage();
        }
    }

    /**
     * Get the causing exception associated with this exception.
     * @return The causing exception or null if no cause is specified.
     */
    public Throwable getCause() {
        return cause;
    }

    /**
     * Prints this DeploymentDescException and its backtrace, and the causes
     * and their stack traces to the standard error stream. 
     */
    public void printStackTrace() {
        super.printStackTrace();
        if (cause != null) {
            System.err.println();
            System.err.println("*** Caused by:");
            cause.printStackTrace();
        }
    }

    /**
     * Prints this DeploymentDescException and its backtrace, and the causes
     * and their stack traces to the e specified print stream. 
     */
    public void printStackTrace(PrintStream s) {
        super.printStackTrace(s);
        if (cause != null) {
            s.println();
            s.println ("*** Caused by:");
            cause.printStackTrace(s);
        }
    }

    /**
     * Prints this DeploymentDescException and its backtrace, and the causes
     * and their stack traces to the e specified print writer. 
     */
    public void printStackTrace(PrintWriter s) {
        super.printStackTrace(s);
        if (cause != null) {
            s.println();
            s.println("*** Caused by:");
            cause.printStackTrace(s);
        }
    }
}
