/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A. 
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Lutris Technologies Inc http://www.lutris.com
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
*/


package org.objectweb.jonas.deployment.api;

import java.lang.Class;

import org.objectweb.jonas.deployment.xml.EjbRef;

/**
 * Class that represents an ejb-ref tag. Only ejb-name and ejb-link are used in Jonas
 * @author Christophe Ney
 */
public class EjbRefDesc {

    String name;
    String jndiName;

    /**
     * Package protected constructor used by API
     */
    EjbRefDesc(EjbRef res) 
            throws DeploymentDescException {
        name = res.getEjbRefName().getContent();
        try {
            jndiName = res.getEjbLink().getContent();
        } catch(NullPointerException e) {
            throw new DeploymentDescException("ejb-link must be set for ejb-ref "+name);
        }
    }

    /**
     * Get the name of the ejb-ref 
     * @return String representation of the name.
     */
    public String getName() {
    	return name;
    }

    /**
     * Get the jndi name of the ejb-ref.
     * @return String representation of the JNDI name
     */
    public String getJndiName() {
        return jndiName;
    }
    
    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\ngetName()="+getName());
        ret.append("\ngetJndiName()="+getJndiName());
        return ret.toString();
    }
    
    
}

