/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A. 
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Lutris Technologies Inc http://www.lutris.com
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
*/


package org.objectweb.jonas.deployment.api;

import java.lang.Class;

import org.objectweb.jonas.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas.deployment.xml.Entity;
import org.objectweb.jonas.deployment.xml.JonasEntity;


/**
 * Base class for descriptor of an entity bean 
 * @author Helene Joanin, Christophe Ney
 */
public abstract class EntityDesc extends BeanDesc {

    Class primaryKeyClass;
    boolean reentrant;    
    protected static final String METHODS_REMOTE_NO_TX = ",getEJBHome,getHandle,getPrimaryKey,isIdentical,";
    protected static final String METHODS_HOME_NO_TX = ",getEJBMetaData,getHomeHandle,";
    
    /**
     * Package protected constructor used by API
     */
    EntityDesc(ClassLoader classLoader,Entity ent, AssemblyDescriptor asd, JonasEntity jEnt)
                throws DeploymentDescException {
        super(classLoader,ent.getDisplayName(), ent.getEjbName(),jEnt.getJndiName(),
              ent.getHome(), ent.getRemote(), ent.getEjbClass(),
              ent.getEnvEntry(),ent.getResourceRef(),jEnt.getJonasResource(),ent.getEjbRef(),asd);

        // primary Key class
        try {
            primaryKeyClass = classLoader.loadClass(ent.getPrimKeyClass().getContent());
        } catch (ClassNotFoundException e) {
            throw new DeploymentDescException("Primary Key class not found for bean "+this.ejbName,e);
        }
        
        // reentrant
        if (ent.getReentrant().getContent().equals("True")) {
            reentrant = true;
        } else if (ent.getReentrant().getContent().equals("False")) {
            reentrant = false;
        } else
            throw new DeploymentDescException("Invalid reentrant value for bean "+this.ejbName);

    }

    /**
     * check that trans-attribute is valid for bean
     */
    protected void checkTxAttribute(java.lang.reflect.Method m) throws DeploymentDescException {
        MethodDesc md = (MethodDesc)methodDesc.get(m);
        if (md.getTxAttribute()==MethodDesc.TX_NOT_SET) {
            // exclude method list for home interface
            if ((m.getClass()==home) && (METHODS_HOME_NO_TX.indexOf(","+m.getName()+",")!=-1))
                return;
            // exclude method list for remote interface
            if ((m.getClass()==remote) && (METHODS_REMOTE_NO_TX.indexOf(","+m.getName()+",")!=-1))
                return;
            // trans-attribute not set !
            throw new DeploymentDescException(" trans-attribute missing for method "+m.toString()+" in session bean "+getEjbName());
        }
    }
    
    /**
     * Get the entity's primary key class.
     * @return Class for the primary key
     */
    public Class getPrimaryKeyClass(){
	    return primaryKeyClass;
    }

    /**
     * Assessor for reentrant entity bean
     * @return true for reentrant entity bean
     * (Not yet used).
     */
    public boolean isReentrant(){
        return reentrant;
    }
    
    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        ret.append("\ngetPrimaryKeyClass()="+getPrimaryKeyClass().toString());
        ret.append("\nisReentrant()="+new Boolean(isReentrant()).toString());
        return ret.toString();
    }
    

}

