/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A. 
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Lutris Technologies Inc http://www.lutris.com
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 * cney 13.3.00
*/


package org.objectweb.jonas.deployment.api;

import java.lang.String;
import java.util.Enumeration;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

import org.objectweb.jonas.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas.deployment.xml.Entity;
import org.objectweb.jonas.deployment.xml.JonasEntity;
import org.objectweb.jonas.deployment.xml.JdbcMapping;
import org.objectweb.jonas.deployment.xml.CmpFieldJdbcMapping;
import org.objectweb.jonas.deployment.xml.FinderMethodJdbcMapping;
import org.objectweb.jonas.deployment.xml.JonasMethod;

/**
 * Base class for descriptor of an entity bean with jdbc based container managed persistence 
 * @author Christophe Ney
 */
public class EntityJdbcCmpDesc extends EntityCmpDesc {


    String jdbcTableName;
    String datasourceJndiName;
    
    /**
     * Package protected constructor used by API
     */
    EntityJdbcCmpDesc(ClassLoader classLoader, Entity ent, AssemblyDescriptor asd, JonasEntity jEnt)
                throws DeploymentDescException {
        super(classLoader,ent,asd,jEnt);
        
        // check for jdbcMapping
        JdbcMapping jm = jEnt.getJdbcMapping();
        if (jm==null)
            throw new DeploymentDescException("jdbc-mapping missing for bean "+ent.getEjbName().getContent());

        // populate field descriptor hashtable with jdbc information
        CmpFieldJdbcMapping cfm[] = jm.getCmpFieldJdbcMapping();
        for (int i=0; i<cfm.length;i++) {
            String fn = cfm[i].getFieldName().getContent();
            FieldDesc f = (FieldDesc)fieldDesc.get(fn);
            if (f==null)
                throw new DeploymentDescException("field-name "+fn+" listed in cmp-field-jdbc-mapping is not of cmp-field of bean "+ent.getEjbName().getContent());
            fieldDesc.put(fn,new FieldJdbcDesc(cfm[i].getJdbcFieldName().getContent(),f.isPrimaryKey()));
        }
        // check that all cmp fields are jdbc
        Enumeration e = fieldDesc.keys();
        while (e.hasMoreElements()) {
            String fn = (String)e.nextElement();
            if (!(fieldDesc.get(fn) instanceof FieldJdbcDesc)) 
                throw new DeploymentDescException("field-name "+fn+" is missing in cmp-field-jdbc-mapping for bean "+ent.getEjbName().getContent());
        }

        // change all methodDesc to jdbcMethodDesc
        e = methodDesc.keys();
        while (e.hasMoreElements()) {
            Method key = (Method)e.nextElement();
            methodDesc.put(key,new MethodJdbcDesc((MethodDesc)methodDesc.get(key)));
        }
        
        // populate method descriptor hashtable with jdbc information
        FinderMethodJdbcMapping fmj[] = jm.getFinderMethodJdbcMapping();
        for (int i=0; i<fmj.length; i++) {
            JonasMethod m = fmj[i].getJonasMethod();
            // loop on methods in the hashtable
            e = methodDesc.keys();
            while (e.hasMoreElements()) {
                java.lang.reflect.Method meth = (java.lang.reflect.Method)e.nextElement();
                // create new descriptor if pattern matches with method                    
                int matchStatus = MethodDesc.matchPattern(meth,null,m.getMethodName(),m.getMethodParams());
                MethodJdbcDesc md = (MethodJdbcDesc)methodDesc.get(meth);
                // set if proper level of assignation
                md.overwriteJdbcWhereClause(fmj[i].getJdbcWhereClause(),matchStatus);
            }
        }

        // modify non jdbc method descriptors
        e = this.methodDesc.keys();
        while (e.hasMoreElements()) {
             java.lang.reflect.Method meth = (java.lang.reflect.Method)e.nextElement();
             MethodDesc md = (MethodDesc)methodDesc.get(meth);
             if (!(md instanceof MethodJdbcDesc)) {
                 this.methodDesc.put(meth,new MethodJdbcDesc(md));
             }
        }
        
        // jndi name of the datasource
        datasourceJndiName = jm.getJndiName().getContent();;
            
        //table name
        jdbcTableName=jm.getJdbcTableName().getContent();
    }

    
    /**
     * Get the datasource jndi name
     * @return String representation of the jndi name
     */
    public String getDatasourceJndiName(){
        return datasourceJndiName;
    }

    /**
     * Assessor for existence of a jdbc descriptor for a given field
     * @param field public field of the bean class
     * @return true if jdbc descriptor is available for given field
     */
    public boolean hasFieldJdbcDesc(Field field) {
        return super.hasCmpFieldDesc(field);        
    }
    
    
    /**
     * Get jdbc specific descriptor for a given field.
     * Used by GenIC
     * @param field of the bean class
     * @return Descriptor for the given field
     */
    public FieldJdbcDesc getFieldJdbcDesc(Field field) {
        return (FieldJdbcDesc)super.getCmpFieldDesc(field);
    }
    

    /**
     * Get the associated DataBase table name in case of container persistence type.
     * Used by GenIC (This information is JOnAS specific).
     * @return Name of the database table where entity bean is stored
     */
    public String getJdbcTableName(){
        return jdbcTableName;
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        ret.append("\ngetDatasourceJndiName()="+getDatasourceJndiName());
        ret.append("\ngetJdbcTableName()="+getJdbcTableName());
        return ret.toString();
    }

}

