/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A. 
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Lutris Technologies Inc http://www.lutris.com
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
* cney 13.3.00
*/

package org.objectweb.jonas.deployment.api;

import org.objectweb.jonas.deployment.xml.JdbcWhereClause;

/**
 * Class for descriptor of JDBC finder methods
 * @author Christophe Ney
 */
public class MethodJdbcDesc extends MethodDesc  {
    
    protected String whereClause = null;
    protected int whereClauseStatus = APPLY_TO_BEAN;
    
    /**
     * package protected copy constructor used by API
     */
    MethodJdbcDesc(MethodDesc object)
            throws DeploymentDescException {
        super(object);
    }
    
    /**
     * Overwrite JdbcWhereClause
     * @param jdbcWhereClause new value for jdbcWhereClause
     * @param status applicability of given transAttribute parameter
     */
    void overwriteJdbcWhereClause(JdbcWhereClause jdbcWhereClause,int status)
            throws DeploymentDescException {
        if (status<this.whereClauseStatus)
            return;
        whereClause = jdbcWhereClause.getContent();
        whereClauseStatus = status;
    }

    /**
     * Assessor for 'WHERE' clause set
     * @return true if where clause has been set
     */
    public boolean hasWhereClause(){
        return whereClause!=null;
    }

    /**
     * Get 'WHERE' clause for the method
     * @return String containing 'WHERE' clause 
     */
    public String getWhereClause(){
        if (whereClause==null) {
            throw new Error("No whereClause defined for this method");
        }
        return whereClause;
    }

    /**
     * Get 'WHERE' clause status for the method
     */
    public int getWhereClauseStatus(){
        return whereClauseStatus;
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        if (hasWhereClause()) {
            ret.append("\ngetWhereClause()="+getWhereClause());
            ret.append("\ngetWhereClauseStatus()="+APPLY_TO[getWhereClauseStatus()]);
        }
        return ret.toString();
    }
    
}
