/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A. 
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Lutris Technologies Inc http://www.lutris.com
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.deployment.api;


import org.objectweb.jonas.deployment.xml.Session;
import org.objectweb.jonas.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas.deployment.xml.JonasSession;


/*
 * Base class for deployment descriptor of session bean
 * @author Helene Joanin, Christophe Ney
 */
public abstract class SessionDesc extends BeanDesc {

    protected int transactionType;

    public final static int BEAN_TRANSACTION_TYPE = 1;
    public final static int CONTAINER_TRANSACTION_TYPE = 2;

    private static final String[] TRANS = {null,"BEAN_TRANSACTION_TYPE","CONTAINER_TRANSACTION_TYPE"};
    
    /**
     * package protected constructor used by API
     */
    SessionDesc(ClassLoader classLoader, Session ses, AssemblyDescriptor asd, JonasSession jSes) 
            throws DeploymentDescException {
        super(classLoader,ses.getDisplayName(), ses.getEjbName(),jSes.getJndiName(),
              ses.getHome(), ses.getRemote(), ses.getEjbClass(),
              ses.getEnvEntry(),ses.getResourceRef(),jSes.getJonasResource(),ses.getEjbRef(),asd);
        // bean managed transaction
        if (ses.getTransactionType().getContent().equals("Bean")) {
            transactionType = BEAN_TRANSACTION_TYPE;
        // container managed transaction
        } else if (ses.getTransactionType().getContent().equals("Container")) {
            transactionType = CONTAINER_TRANSACTION_TYPE;
        // error
        } else {
            throw new DeploymentDescException("Invalid transaction-type content for ejb-name "+ejbName);
        }
    }

    /**
     * check that trans-attribute is valid for bean
     */
    protected void checkTxAttribute(java.lang.reflect.Method m) throws DeploymentDescException {
        MethodDesc md = (MethodDesc)methodDesc.get(m);
        if (getTransactionType()==CONTAINER_TRANSACTION_TYPE) {
            // tx-attribute must be set for remote interface
            if ((md.getTxAttribute()==MethodDesc.TX_NOT_SET) && (m.getClass()==remote))
                throw new DeploymentDescException(" trans-attribute missing for method "+m.toString()+" in session bean "+getEjbName());
        } else {
            if (md.getTxAttribute()!=MethodDesc.TX_NOT_SET)
                throw new DeploymentDescException(md.getTxAttributeName()+" is not a valid trans-attribute for method "+m.toString()+" in session bean "+getEjbName());
        }
    }
        
    /**
     * Get session transaction management type.<BR>
     * Used by Jonas server
     * @return transaction type value within BEAN_TRANSACTION_TYPE,CONTAINER_TRANSACTION_TYPE
     */
    public int getTransactionType(){
        return transactionType;
    }
    
    /**
     * Check that the bean descriptor is valid
     * @exception DeploymentDescException thrown for non-valid bean
     */
    public void check() throws DeploymentDescException {
        super.check();
        // for BEAN_TRANSACTION_TYPE transactions ejbClass should not implement javax.ejb.SessionSynchronization
        if ((getTransactionType()==BEAN_TRANSACTION_TYPE) && (javax.ejb.SessionSynchronization.class.isAssignableFrom(ejbClass)))
            throw new DeploymentDescException(ejbClass.getName()+" should NOT manage transactions and implement javax.ejb.SessionSynchronization");
    }
    
    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        ret.append("\ngetTransactionType()"+TRANS[getTransactionType()]);
        return ret.toString();
    }
    
}
