/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jtm;

import java.util.Vector;
import java.rmi.RemoteException;
import org.objectweb.jonas.common.RemoteObject;
import org.objectweb.jonas.common.Trace;

public class TransactionFactoryImpl extends RemoteObject implements TransactionFactory {

    /**
     * @serial
     */
    long timeoutMax = 3600;	// 1 hour

    static int trace = Trace.DB_4;

    private Vector coordinatorList = new Vector();

    /**
     * Constructor of the Transaction Factory
     */
    public TransactionFactoryImpl() throws RemoteException {
	Trace.outln(trace, "TransactionFactory constructor");
    }
    
    /**
     * Create a new Transaction.
     *
     * @return		The Control object for the transaction
     */
    public synchronized Control create(long timeout) throws RemoteException {
	Trace.outln(trace, "TransactionFactoryImpl.create("+timeout+")");

	ControlImpl ctrl = null;

	// checks timeout value
	if (timeout == 0 || timeout > timeoutMax)
	    timeout = timeoutMax;

	// makes a new otid
	// - should pass servername + ip addr. (LATER)
	Otid otid = new Otid("TMServer", 0);

	// Creates a new ControlImpl
	try {
	    ctrl = new ControlImpl(timeout, otid, null);
	} catch(Exception e) {
	    Trace.errln("Cannot create ControlImpl" + e);
	}
	return ctrl;
    }

    /**
     * Search in the coordinator list if a transaction matching this propagation
     * context exists.
     *
     * @return		The ControlImpl object, or null if not found.
     */
    ControlImpl getControl(PropagationContext ctx) {
	Trace.outln(trace, "TransactionFactoryImpl: getControl");

	ControlImpl ret = null;

	synchronized (coordinatorList) {
	    for (int i = 0; i < coordinatorList.size(); i++) {
		Coordinator coord = (Coordinator) coordinatorList.elementAt(i);
		if (coord.equals(ctx.getCoordinator())) {
		    Trace.outln(trace, "getControl: Control already in the list");
		    ret = (ControlImpl) coord;
		    break;
		}
	    }
	}
	return ret;
    }

    /**
     * Recreate locally a Control object for an existing transaction. It is
     * possible to call recreate for a transaction already known. In this
     * case, recreate simply returns the existing Control object.
     *
     * @return		The Control object for the transaction
     */
    public synchronized Control recreate(PropagationContext ctx) throws RemoteException {

	Trace.outln(trace, "recreate");

	// Check if Control already exists
	ControlImpl ctrl = getControl(ctx);
	if (ctrl != null) {
	    Trace.outln(trace, "recreate twice");
	    return ctrl;
	}

	// Build an otid with same gtrid and a new bqual
	// - See create for args.
	Otid otid = new Otid(ctx.getXid(), "TMServer", 0);

	// Creates a new ControlImpl and register it to the sup-coord.
	Coordinator supco = ctx.getCoordinator();
	try {
	    ctrl = new ControlImpl(ctx.getTimeout(), otid, supco);
	} catch(Exception e) {
	    Trace.errln("Cannot create ControlImpl" + e);
	}

	return ctrl;
    }

}
