/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.lib; 

import java.io.File;
import java.util.*;

import org.objectweb.jonas.deployment.api.BeanDesc;

/**
 * This class is made for hiding Naming convention in all JOnAS framework.
 * <br>
 * (Among other things, names of the implementation classes of
 * the Enterprise Bean's Home and Enterprise Bean Remote interfaces
 * generated by the GenIC tools.)
 * <br>
 * It is used by the EJB generation tools and framework
 */

public class BeanNaming {

    
    private String fullWrpRemoteName;
    private String wrpRemoteName;

    private String fullWrpHomeName;
    private String wrpHomeName;

    private String fullWrpHandleName;
    private String wrpHandleName;


    /**
     * Create a BeanNaming Object from the Deployment Descriptor.
     * <br>
     * (Generally speaking FullXName means packageName + Name X)
     */
    public BeanNaming(BeanDesc dd){
	String s, p;
	String b = dd.getEjbClass().getName();

	s = dd.getRemoteClass().getName();
	p = getPackageName(s);
	wrpRemoteName     = new String("JOnAS"+getBaseName(b)+getBaseName(s));
	wrpHandleName     = new String("JOnAS"+getBaseName(b)+getBaseName(s)+"Handle");
	if (p.equals("")) {
	    fullWrpRemoteName = new String(wrpRemoteName);
	    fullWrpHandleName = new String(wrpHandleName);
	} else {
	    fullWrpRemoteName = new String(p+"."+wrpRemoteName);
	    fullWrpHandleName = new String(p+"."+wrpHandleName);
	}

	s = dd.getHomeClass().getName();
	p = getPackageName(s);
	wrpHomeName     = new String("JOnAS"+getBaseName(b)+getBaseName(s));
	if (p.equals("")) {
	    fullWrpHomeName = new String(wrpHomeName);
	} else {
	    fullWrpHomeName = new String(p+"."+wrpHomeName);
	}

    }


    /**
     * returns the name of the package of the given full name
     */
    public static String getPackageName(String fullName) {
	String packageName;
	StringTokenizer stk = new StringTokenizer(fullName, ".");
	int nb = stk.countTokens();
	packageName = new String();
	for (int i=0; i < nb-1; i++) {
	    packageName = packageName.concat(stk.nextToken());
	    if (i < nb-2) {
		packageName = packageName.concat(".");
	    }
	}	
	return packageName;
    }

    /**
     * returns the basename of the given full name
     */
    public static String getBaseName(String fullName) {
	String packageName, baseName;
	StringTokenizer stk = new StringTokenizer(fullName, ".");
	int nb = stk.countTokens();
	packageName = new String();
	for (int i=0; i < nb-1; i++) {
	    packageName = packageName.concat(stk.nextToken());
	    if (i < nb-2) {
		packageName = packageName.concat(".");
	    }
	}
	if (nb > 0) {
	    baseName = new String(stk.nextToken());
	} else {
	    baseName = new String();
	}
	return baseName;
    }

    /**
     * returns the given string with the first character converted to upper case
     */
    public static String firstToUpperCase(String s) {
	String value;
	if (s.length() > 0) {
	    char c = Character.toUpperCase(s.charAt(0));
	    value = new String(c+s.substring(1));
	} else {
	    value = new String();
	}
	return(value);
    }

    /**
     * returns the full name of the wrapper class for the bean
     */
    public String getFullWrpRemoteName(){
	return fullWrpRemoteName;
    }

    /**
     * returns the name of the wrapper class for the bean
     */
    public String getWrpRemoteName(){
	return wrpRemoteName;
    }

    /**
     * returns the full name of the Handle class for the bean
     */
    public String getFullWrpHandleName(){
	return fullWrpHandleName;
    }

    /**
     * returns the name of the Handle class for the bean
     */
    public String getWrpHandleName(){
	return wrpHandleName;
    }

    /**
     * returns the full name of the wrapper home class for the bean
     */
    public String getFullWrpHomeName(){
	return fullWrpHomeName;
    }

    /**
     * returns the name of the wrapper home class for the bean
     */
    public String getWrpHomeName(){
	return wrpHomeName;
    }


    /**
     * returns the name of the JOnAS specific deployment descriptor file's name
     * corresponding to the given standard deployment descriptor file's name.
     * (ex: returns "jonas-XXX.xml" for "XXX.xml" and "../../jonas-XXX.xml" for "../../XXX.xml")
     */
    public static String getJonasXmlName(String stdXmlName) {

	String jonasXmlName = new String();
	File f = new File(stdXmlName);
	String p = f.getParent();
	String n = f.getName();
	if (p != null) {
	    jonasXmlName = jonasXmlName.concat(p);
	    jonasXmlName = jonasXmlName.concat(File.separator);
	}
	jonasXmlName = jonasXmlName.concat("jonas-");
	jonasXmlName = jonasXmlName.concat(n);

	return(jonasXmlName);
    }

}

