/*
 *
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.naming.java;

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.spi.ObjectFactory;
import org.objectweb.jonas.common.Trace;

/**
 * Context factory for javaURLContext objects.
 * This factory will be used for all "java:..." urls provided as Name objects
 * for all JNDI operations.
 */
public class javaURLContextFactory implements ObjectFactory {

    static private final int trace = Trace.DB_3;

    /**
     * Returns an instance of javaURLContext for a java URL.
     *
     * If url is null, the result is a context for resolving java URLs.
     * If url is a URL, the result is a context named by the URL.
     *
     * @param url 	String with a "java:" prefix or null.
     * @param name	Name of context, relative to ctx, or null.
     * @param ctx	Context relative to which 'name' is named.
     * @param env	Environment to use when creating the context
     */
    public Object getObjectInstance(Object url, Name name, Context ctx, Hashtable env) throws Exception {

	Trace.outln(trace, "javaURLContextFactory.getObjectInstance:" + url);

	if (url == null) {
	    // All naming operations with "java:..." comes here
	    // Users are encouraged to used intermediate contexts:
	    // ctx = ic.lookup("java:comp/env") called only once (perfs)
	    return new javaURLContext(env);
	}
	if (url instanceof String) {
	    // Don't know what to do here 
	    Trace.outln(Trace.DB_31, "javaURLContextFactory.getObjectInstance("+url+")");
	    return null;
	} else if (url instanceof String[]) {
	    // Don't know what to do here 
	    Trace.outln(Trace.DB_31, "javaURLContextFactory.getObjectInstance(String[])");
	    return null;
	} else {
	    // invalid argument
	    throw (new IllegalArgumentException("javaURLContextFactory"));
	}
    }
}
