package org.objectweb.jonas.rmifilters;

import org.omg.CosTSPortability.Receiver;
import org.omg.CosTransactions.PropagationContext;

import org.objectweb.jonathan.apis.kernel.Kernel;
import org.objectweb.jonathan.apis.kernel.JonathanException;
import org.objectweb.jonathan.apis.resources.Scheduler;

import org.objectweb.jonas.jtm.Current;
import org.objectweb.jonas.common.Trace;

public class JonasReceiver implements Receiver {
   
   static final int trace = Trace.DB_6;

   static JonasReceiver[] instances = new JonasReceiver[3];
   static int instance_nb = 0;

   Kernel kernel; 

   protected JonasReceiver(Kernel kernel) throws JonathanException {
      Trace.outln(trace, "initialising a JonasReceiver");
      this.kernel = kernel;
   }
   
   public static JonasReceiver init(Kernel kernel)
      throws JonathanException {
         Scheduler scheduler = (Scheduler)
            kernel.bind("SCHEDULER");
         try {
            scheduler.lock(JonasReceiver.class);
            for (int i = 0; i < instance_nb; i++) {
               if (instances[i].kernel.equals(kernel)) {
                  return instances[i];
               }
            }
            JonasReceiver instance = new JonasReceiver(kernel);
            if (instance_nb == instances.length) {
               JonasReceiver[] new_instances = new JonasReceiver[instance_nb + 3];
               System.arraycopy(instances,0,new_instances,0,instance_nb);
               instances = new_instances;
            }
            instances[instance_nb++] = instance;
            return instance;
         } finally {
            scheduler.unlock(JonasReceiver.class);
         }
   }
  
  public void received_request (int reqId, PropagationContext pctx) {
     Trace.outln(trace, "received_request() " + this + " context " + pctx);
     // Gets Current object (always existing in EJBServer)
     Current ut = Current.getCurrent();
     if (ut == null) {
        Trace.errln("No Current in this server!");
        return;
     }
     // Associates the current thread with this propagation context
     if (pctx != null) {
        Trace.outln(trace, "  received_request: valid transaction context");
        Trace.outln(trace, "  Coordinator of this pCtxt is " +
         ((org.objectweb.jonas.jtm.PropagationContext)pctx).getCoordinator());
     }
     ut.setPropagationContext
        ((org.objectweb.jonas.jtm.PropagationContext) pctx);
  }
  

  public PropagationContext sending_reply (int reqId) {
     // Gets Current object (always existing in EJBServer)
     Current ut = Current.getCurrent();     
     if (ut == null) {
        Trace.errln("No Current in this server!");
        return null;
        //throw new RuntimeException("No Current in this server");
     }
     // Get the Propagation Context
     PropagationContext pctx = ut.getPropagationContext();
     // Dissociates the current thread from any propagation context
     if (pctx != null) {
        ut.setPropagationContext(null);
     }
     Trace.outln(trace, "sending_reply() : creates reply context " + pctx);
     if (pctx != null) {
        Trace.outln(trace, "  Coordinator of this pCtxt is " +
           ((org.objectweb.jonas.jtm.PropagationContext)pctx).getCoordinator());
     }
     return pctx;
  }
   
}
