package org.objectweb.jonas.rmifilters;

import org.omg.CosTSPortability.Sender;
import org.omg.CosTransactions.PropagationContext;
import org.omg.CORBA.Environment;

import org.objectweb.jonathan.apis.kernel.Kernel;
import org.objectweb.jonathan.apis.kernel.JonathanException;
import org.objectweb.jonathan.apis.resources.Scheduler;

import org.objectweb.jonas.jtm.Current;
import org.objectweb.jonas.common.Trace;

public class JonasSender implements Sender {

   static final int trace = Trace.DB_6;
   
   static JonasSender[] instances = new JonasSender[3];
   static int instance_nb = 0;

   Kernel kernel; 

   protected JonasSender(Kernel kernel) throws JonathanException {
      Trace.outln(trace, "initialising a JonasSender");
      this.kernel = kernel;
   }
   
   public static JonasSender init(Kernel kernel)
      throws JonathanException {
         Scheduler scheduler = (Scheduler)
            kernel.bind("SCHEDULER");
         try {
            scheduler.lock(JonasSender.class);
            for (int i = 0; i < instance_nb; i++) {
               if (instances[i].kernel.equals(kernel)) {
                  return instances[i];
               }
            }
            JonasSender instance = new JonasSender(kernel);
            if (instance_nb == instances.length) {
               JonasSender[] new_instances = new JonasSender[instance_nb + 3];
               System.arraycopy(instances,0,new_instances,0,instance_nb);
               instances = new_instances;
            }
            instances[instance_nb++] = instance;
            return instance;
         } finally {
            scheduler.unlock(JonasSender.class);
         }
   }

   
   public PropagationContext sending_request (int reqId) {
      // Get the Propagation Context
      PropagationContext pctx = null;
      Current ut = Current.getCurrent();
      if (ut != null) {
         pctx = ut.getPropagationContext();
      }
      Trace.outln(trace, "sending_request()   creates request context " + pctx);
      if (pctx != null) {
         Trace.outln(trace, "  Coordinator of this pCtxt is " +
         ((org.objectweb.jonas.jtm.PropagationContext) pctx).getCoordinator());
      }
      return pctx;
   }

  public void received_reply (int reqId, PropagationContext pctx,
                              Environment env) {
     // Associates the current thread with this propagation context
     Trace.outln(trace, "received_reply  context " + pctx);
     if (pctx != null) {
        Trace.outln(trace, "  Coordinator of this pCtxt is " +
           ((org.objectweb.jonas.jtm.PropagationContext) pctx).getCoordinator());
        Current ut = Current.getCurrent();
        if (ut != null) {
           ut.setPropagationContext
              ((org.objectweb.jonas.jtm.PropagationContext) pctx);
        }
     }
  }
   
}
