/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.server;

import java.rmi.RMISecurityManager;
import java.util.Vector;
import javax.naming.Context;
import javax.naming.NamingException;
import org.objectweb.jonas.adm.Adm;
import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.container.EJBServiceImpl;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.jtm.TransactionServiceImpl;
import org.objectweb.jonas.naming.CompNamingContext;
import org.objectweb.jonas.naming.NamingManager;
import org.objectweb.jonas.dbm.DataBaseManager;

/**
 * This class implements an EJB Server.
 *  Only one method: main().
 */
public class Server {

    static private final int trace = Trace.DB_14;

    static Adm adm = null;
    static NamingManager namingManager = null;
    static TransactionService transactionManager = null;

    /**
     * EJBServer main routine. No args.
     */
    public static void main(String args[]) {

	// Print jonas version and orb
	String orbs = null;
	if (Env.isJeremie()) {
	    orbs = "Jeremie.";
	} else {
	    orbs = "rmi.";
	}
	System.out.println("JOnAS Server, version 2.0, running on " + orbs);

	// RMI Security Maqnager
	if (System.getSecurityManager() == null) {
	    System.setSecurityManager(new RMISecurityManager());
	}

	// Init Trace system
	Trace.configure();

	// Start the NamingManager now to see quickly if there is
	// a problem with naming.
	try {
	    NamingManager.getInstance();
	} catch (Exception e) {
	    Trace.errln("JOnAS: Cannot start Naming Manager");
	    System.exit(2);	
	}

	// Start the Transaction Service
	try {
	    Context ctx = new CompNamingContext("TransactionService");
	    ctx.rebind("remote", Prop.svr.getValue("jonas.tm.remote"));
	    TransactionServiceImpl.getInstance().start(ctx);
	} catch (Exception e) {
	    Trace.errln("JOnAS: Cannot start Transaction Service");
	    System.exit(2);	
	}

	// Start the DataBaseManager
	try {
	    DataBaseManager.getInstance().start();
	} catch (Exception e) {
	    Trace.errln("JOnAS: Cannot start DataBase Manager");
	    System.exit(2);	
	}
 
	// Start the EJBService
	try {
	    Context ctx = new CompNamingContext("EJBService");
	    ctx.rebind("descriptors", Prop.svr.getValue("jonas.beans.descriptors", null));
	    EJBServiceImpl.getInstance().start(ctx);
	} catch(Exception e) {
	    Trace.errln ("Cannot create containers : " + e);
	    System.exit(2);	
	}

	// Creates the Adm instance for remote administration 
	try {
	    adm =  new Adm(Env.getJonasName());
	} catch(Exception e) {
	    Trace.errln ("Cannot create Adm : " + e);
	    System.exit(2);	
	}
	Trace.outln(Trace.DB_31, "EJBServer is ready");
    }

}
