/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.server;

import java.io.*;
import javax.naming.Context;
import javax.naming.NamingException;
import org.objectweb.jonas.adm.Adm;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.jtm.TransactionServiceImpl;
import org.objectweb.jonas.naming.CompNamingContext;
import org.objectweb.jonas.naming.NamingManager;

/**
 * TMServer
 */
public class TMServer {

    static private final int trace = Trace.DB_4;

    static Adm adm = null;
    static NamingManager namingManager = null;
    static TransactionService transactionService = null;

    /**
     * TMServer main routine. No args.
     */
    public static void main(String[] args) {
	
	Trace.configure();

	// Start the NamingManager now to see quickly if there is
	// a problem with naming.
	try {
	    NamingManager.getInstance();
	} catch (Exception e) {
	    Trace.errln("TMServer: Cannot start Naming Manager");
	    System.exit(2);	
	}

	// Start the Transaction manager
	try {
	    Context ctx = new CompNamingContext("TransactionService");
	    ctx.rebind("remote", "true");
	    TransactionServiceImpl.getInstance().start(ctx);
	} catch (Exception e) {
	    Trace.errln("TMServer: Cannot start Transaction Manager");
	    e.printStackTrace();
	    System.exit(2);	
	}

	// Creates the Adm instance for remote administration 
	// NamingManager must already exist.
	try {
	    
	    adm =  new Adm("TMServer");
	} catch(Exception e) {
	    Trace.errln("Cannot create Adm : " + e);
	    e.printStackTrace();
	    System.exit(2);	
	}
	Trace.outln(Trace.DB_31, "TM is ready");
    }
}
