/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;

import java.io.IOException;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.IndentedFile;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.BeanDesc;

abstract class GenICCommon {

    static final int trace = Trace.DB_19;

    IndentedFile src = null;

    GenICCommon(BeanDesc dd, String fileName) throws GenICException {

	// Source File
	try {
	    src = new IndentedFile(fileName);
	} catch (IOException e) {
	    Trace.errln(e, true);
	    throw new GenICException("Cannot create '" + fileName + "' file");
	}
    }

    /**
     * Source generation of the 'catch and finally' code following any call of a bean method
     * (such bussiness methods, ejbCreate(), ejbPostCreate(),, .....)
     * @param mInter    the current generated method of the interface
     */
    void genCatchBean(Method mInter) {
	// The "try { ....;" has already been generated
	// The "}" will be generated
	src.indentMinus();
	src.println("} catch (RuntimeException e) {");
	src.indentPlus();
	src.println("rctx.setSysExc(e);");
	src.println("throw new RemoteException(\"RuntimeException thrown by an enterprise Bean\", e);");
	src.indentMinus();
	src.println("} catch (Error e) {");
	src.indentPlus();
	src.println("rctx.setSysExc(e);");
	src.println("throw new RemoteException(\"Error thrown by an enterprise Bean\", e);");
	src.indentMinus();
    }
}
