/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.EntityBmpDesc;
import org.objectweb.jonas.lib.BeanNaming;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Entity Enterprise Java Bean
 *  with bena managed persistence.
 *  Its extends GenICHome class.
 */

class GenICEntityBMHome extends GenICEntityHome {

    /**
     * GenICEntityHome Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     *                 with bean managed persistence
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityBMHome(EntityBmpDesc dd, String fileName) throws GenICException {
	super(dd, fileName);
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.Object;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import java.util.Enumeration;");
	src.println("import javax.ejb.EJBObject;");
	src.println("import javax.ejb.FinderException;");
	src.println("import javax.ejb.ObjectNotFoundException;");
	src.println("import org.objectweb.jonas.container.JEntityHome;");
	src.println("import org.objectweb.jonas.container.EntityContextImpl;");
	src.println("import org.objectweb.jonas.container.RequestCtx;");
	src.println("import org.objectweb.jonas.deployment.api.EntityDesc;");
	src.println("import org.objectweb.jonas.lib.CollectionEnum;");
	src.println("import org.objectweb.jonas.common.Trace;");
    }

    /**
     * Source generation for the given create() method described in the Home interface
     * @exception GenICException in error case
     */
    void genCreate(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();

	genHeaderMethod(method);
	src.println(wrpRemoteBaseName + " ejbobj;");
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println(ejbBeanName + " eb = null;");
	src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+");");
	src.println("EntityContextImpl ctx = null;");
	src.println(ejbPrimaryKeyName + " pk;");
	src.println("try {");
	src.indentPlus();
	src.println("ctx = ejbobj.createContext();");
	src.println("eb = ("+ejbBeanName +")ctx.getInstance();");
	src.print  ("pk = eb.ejbCreate(");
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	src.println("ejbobj.setPrimaryKey(pk);");
	src.print  ("eb.ejbPostCreate(");
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	genCatchBean(method);
	src.println("} catch(RemoteException e) {");
	src.indentPlus();
	src.println("rctx.setSysExc(e);");
	src.println("throw e;");
	src.indentMinus();
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	src.println("putEJBObject(pk, ejbobj);");
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given findByPrimaryKey() method described in the Home interface
     * @exception GenICException in error case
     */
    void genFindByPK(Method method) throws GenICException {

	super.genFindByPK(method);

	Class params[] = method.getParameterTypes();
	Class exceptions[] = method.getExceptionTypes();

	/*
	 * findByPrimaryKey(PrimaryKey p1, boolean callEjbFindByPK) Method
	 */
	src.print  ("synchronized public " +
		    (JavaType.getName(method.getReturnType())) +
		    " findByPrimaryKey(");
	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]) + " " + "p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.print(", boolean callEjbFindByPK) ", false);
	for (int e=1; e <= exceptions.length; e++) {
	    if (e==1) {
		src.print("throws ", false);
	    }
	    src.print(exceptions[e-1].getName(), false);
	    if (e < exceptions.length) {
		src.print(",", false);
	    }
	    src.print(" ", false);
	}
	src.println("{", false);
	// begining of the core
	src.indentPlus();
	src.println(wrpRemoteBaseName + " ejbobj = (" +
		    wrpRemoteBaseName + ")getEJBObject(p1);");
	src.println("if (ejbobj != null) {");
	src.indentPlus();
	src.println("return((" + ejbRemoteName+ ") ejbobj);");
	src.indentMinus();
	src.println("}");
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println(ejbBeanName + " eb = null;");
	src.println("EntityContextImpl ctx = null;");
	src.println("try {");
	src.indentPlus();
	src.println("ctx = ejbobj.createContext();");
	src.println("eb = ("+ejbBeanName +")ctx.getInstance();");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to create a new instance of the bean class "+ejbBeanName+"\");");
	src.println("Trace.errln(e);");
	src.println("throw new RemoteException(\"Failed to create a new instance of the bean class "+ejbBeanName+"\", e);");
	src.indentMinus();
	src.println("}");
	src.println(ejbPrimaryKeyName + " pk;");
	src.println("if (callEjbFindByPK) {");
	src.indentPlus();
	src.print  ("pk = eb.ejbFindByPrimaryKey(");
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	src.indentMinus();
	src.println("} else {");
	src.indentPlus();
	src.println("pk = p1;");
	src.indentMinus();
	src.println("}");
	src.println("ejbobj.setPrimaryKey(pk);");
	src.println("putEJBObject(pk, ejbobj);");
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given findXXX() method described in the Home interface
     * @exception GenICException in error case
     */
    void genFindOther(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();
	boolean isWithCollection = ! method.getReturnType().equals(ejbRemoteClass);

	genHeaderMethod(method);
	if (!isWithCollection) {
	    src.println(ejbRemoteName + " ejbobj = null;");
	    src.println(ejbBeanName + " eb = null;");
	    src.println("try {");
	    src.indentPlus();
	    src.println("eb = ("+ejbBeanName +")getAvailable();");
	    src.indentMinus();
	    src.println("} catch (Exception e) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to create a new instance of the bean class "+ejbBeanName+"\");");
	    src.println("Trace.errln(e);");
	    src.println("throw new RemoteException(\"Failed to create a new instance of the bean class "+ejbBeanName+"\", e);");
	    src.indentMinus();
	    src.println("}");
	    src.println(ejbPrimaryKeyName + " pk;");
	    src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+");");
	    src.println("try {");
	    src.indentPlus();
	    src.print("pk = eb.ejb" + BeanNaming.firstToUpperCase(method.getName()) + "(");
	    for (int p=1; p <= params.length; p++) {
		src.print("p" + p, false);
		if (p < params.length) {
		    src.print(", ", false);
		}
	    }
	    src.println(");", false);
	    src.println("ejbobj = findByPrimaryKey(pk, false);");
	    genCatchBean(method);
	    src.println("} catch(RemoteException e) {");
	    src.indentPlus();
	    src.println("rctx.setSysExc(e);");
	    src.println("throw e;");
	    src.indentMinus();
	    src.println("} finally {");
	    src.indentPlus();
	    src.println("postinvoke(rctx);");
	    src.indentMinus();
	    src.println("}");
	    src.println("return(ejbobj);");
	} else {
	    // findXXX() method with a Collection
	    boolean isEnumerationReturnType = method.getReturnType().equals(java.util.Enumeration.class);
	    if (isEnumerationReturnType) {
		src.println("CollectionEnum ejbobjV = new CollectionEnum();");
	    } else {
		// java.util.Collection
		src.println("java.util.Vector ejbobjV = new java.util.Vector();");
	    }
	    src.println(ejbRemoteName + " ejbobj = null;");
	    src.println(ejbBeanName + " eb = null;");
	    src.println("try {");
	    src.indentPlus();
	    src.println("eb = ("+ejbBeanName +")getAvailable();");
	    src.indentMinus();
	    src.println("} catch (Exception e) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to create a new instance of the bean class "+ejbBeanName+"\");");
	    src.println("Trace.errln(e);");
	    src.println("throw new RemoteException(\"Failed to create a new instance of the bean class "+ejbBeanName+"\", e);");
	    src.indentMinus();
	    src.println("}");
	    if (isEnumerationReturnType) {
		src.println("Enumeration pkC = null;");
	    } else {
		src.println("java.util.Collection pkC = null;");
	    }
	    src.println(ejbPrimaryKeyName + " pk;");
	    src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+");");
	    src.println("try {");
	    src.indentPlus();
	    src.print  ("pkC = eb.ejb" +
			BeanNaming.firstToUpperCase(method.getName()) +
			"(");
	    for (int p=1; p <= params.length; p++) {
		src.print("p" + p, false);
		if (p < params.length) {
		    src.print(", ", false);
		}
	    }
	    src.println(");", false);
	    if (isEnumerationReturnType) {
		src.println("while (pkC.hasMoreElements()) {");
		src.indentPlus();
		src.println("ejbobj = findByPrimaryKey(("+
			    ejbPrimaryKeyName+")pkC.nextElement(), false);");
		src.println("ejbobjV.addElement((Object)ejbobj);");
		src.indentMinus();
		src.println("}");
	    } else {
		src.println("java.util.Iterator pkIC = pkC.iterator();");
		src.println("while (pkIC.hasNext()) {");
		src.indentPlus();
		src.println("ejbobj = findByPrimaryKey(("+
			    ejbPrimaryKeyName+")pkIC.next(), false);");
		src.println("ejbobjV.addElement((Object)ejbobj);");
		src.indentMinus();
		src.println("}");
	    }
	    genCatchBean(method);
	    src.println("} catch(RemoteException e) {");
	    src.indentPlus();
	    src.println("rctx.setSysExc(e);");
	    src.println("throw e;");
	    src.indentMinus();
	    src.println("} finally {");
	    src.indentPlus();
	    src.println("postinvoke(rctx);");
	    src.indentMinus();
	    src.println("}");
	    src.println("return(ejbobjV);");
	}
	src.indentMinus();
	src.println("}");
    }

}

