/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;

import java.lang.String;
import org.objectweb.jonas.deployment.api.EntityBmpDesc;

/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Entity Enterprise Java Bean
 *  with bean managed persistence.
 *  Its extends GenICEntityRemote class, itself extends GenICRemote class. 
 */

class GenICEntityBMRemote extends GenICEntityRemote {

    /**
     * GenICEntityBMRemote Constructor
     *
     * @param dd       deployment descriptor of the Entity Bean with bean managed persistence
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityBMRemote(EntityBmpDesc dd, String fileName) throws GenICException {
	super(dd, fileName);
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import javax.ejb.EnterpriseBean;");
	src.println("import javax.ejb.EJBException;");
	src.println("import javax.ejb.RemoveException;");
	src.println("import javax.ejb.EntityBean;");
	src.println("import org.objectweb.jonas.container.EntityContextImpl;");
	src.println("import org.objectweb.jonas.container.RequestCtx;");
	src.println("import org.objectweb.jonas.container.JEntityHome;");
	src.println("import org.objectweb.jonas.container.JBeanEntity;");
	src.println("import org.objectweb.jonas.common.Trace;");
    }

    /**
     * Source generation of the createData(), removeData(), loadData(), storeData() methods
     * In case of an entity bean managed persistence, those methods are empty.
     * @exception GenICException in error case
     */
    void genAccessData() throws GenICException {
	// No persistence Container Managed
	// Generate empty methods: createData(), removeData(), loadData(), storeData()
	// Generate method: isModifiedData() which return always true
	src.println("public boolean isModifiedData(EntityBean bean) {");
	src.indentPlus();
	src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".isModifiedData(EntityBean) --> true\");");
	src.println("return(true);");
	src.indentMinus();
	src.println("}");
	src.println("public void createData() {");
	src.println("}");
	src.println("public void removeData() {");
	src.println("}");
	src.println("public void loadData() {");
	src.println("}");
	src.println("public void storeData() {");
	src.println("}");

    }

}

