/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;

import java.lang.String;
import java.lang.reflect.Field;
import org.objectweb.jonas.deployment.api.EntityCmpDesc;
import org.objectweb.jonas.lib.JavaType;
import org.objectweb.jonas.common.Trace;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Entity Enterprise Java Bean
 *  with container managed persistence.
 *  Its extends GenICEntityRemote class, itself extends GenICRemote class.
 */

abstract class GenICEntityCMRemote extends GenICEntityRemote {

    /**
     * GenICEntityCMRemote Constructor
     *
     * @param dd       deployment descriptor of the Entity Bean with container managed persistence
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityCMRemote(EntityCmpDesc dd, String fileName) throws GenICException {

	super(dd, fileName);

    }

    /**
     * Source generation of the isModifiedData() method.
     * The source generation of the createData(), removeData(), loadData(), storeData() methods
     * is done in the Sub-class.
     * @exception GenICException in error case
     */
    void genAccessData() throws GenICException {
	src.println("public boolean isModifiedData(EntityBean bean) {");
	src.indentPlus();
	if (((EntityCmpDesc)ddesc).hasIsModifiedMethod()) {
	    String isModifiedMethodName = ((EntityCmpDesc)ddesc).getIsModifiedMethod().getName();
	    src.println("boolean b = (("+ejbBeanName+")bean)."+isModifiedMethodName+"();");

	} else {
	    src.println("boolean b = true;");
	}
	src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".isModifiedData(EntityBean) --> \" + b);");
	src.println("return(b);");
	src.indentMinus();
	src.println("}");
	src.println();
    }

}

