/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.EntityDesc;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Entity Enterprise Java Bean.
 *  Its extends GenICHome class.
 */
abstract class GenICEntityHome extends GenICHome {

    // Full names of the interfaces and the classes
    String ejbPrimaryKeyName = null;

    /**
     * GenICEntityHome Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityHome (EntityDesc dd, String fileName) throws GenICException {
	super(dd, fileName);

	ejbPrimaryKeyName = dd.getPrimaryKeyClass().getName();
    }

    /**
     * Name of the superclass of the generated class
     */
    String getSuperClass() {
	return(new String("JEntityHome"));
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {

	src.println("public " + wrpHomeBaseName +
		    "(EntityDesc desc, String BeanName, Object ds) throws RemoteException {");
	src.indentPlus();
	src.println("super(desc, BeanName, ds);");
	src.println("Trace.outln(Trace.DB_15, \"" + wrpHomeBaseName + "(\"+BeanName+\")\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given findByPrimaryKey() method described in the Home interface
     * and for findByPK(Object pk) used by the generic container
     * @exception GenICException in error case
     */
    void genFindByPK(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();
	Class exceptions[] = method.getExceptionTypes();

	/*
	 * XXX findByPrimaryKey(PrimaryKey p1) Method
	 */
	src.print("public " + JavaType.getName(method.getReturnType()) +
		  " findByPrimaryKey(");
	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]) + " " + "p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.print  (") ", false);
	for (int e=1; e <= exceptions.length; e++) {
		    if (e==1) {
		src.print("throws ", false);
	    }
	    src.print(exceptions[e-1].getName(), false);
	    if (e < exceptions.length) {
		src.print(",", false);
	    }
	    src.print(" ", false);
	}
	src.println("{", false);
	// begining of the core
	src.indentPlus();
	src.println(JavaType.getName(method.getReturnType()) + " ret = null;");
	int transAttribute = ddesc.getMethodDesc(method).getTxAttribute();
	src.println("RequestCtx rctx = preinvoke(" + transAttribute + ");");
	src.println("try {");
	src.indentPlus();
	src.println("ret = findByPrimaryKey(p1, true);");
	src.indentMinus();
	src.println("} catch (RemoteException e) {");
	src.indentPlus();
	src.println("rctx.setSysExc(e);");
	src.println("throw e;");
	genCatchBean(method);
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	src.println("return ret;");
	src.indentMinus();
	src.println("}");
	src.println();

	/*
	 * Object findByPK(Object p1) Method
	 */
	src.print("public java.lang.Object findByPK(java.lang.Object  p1) ");
	for (int e=1; e <= exceptions.length; e++) {
	    if (e==1) {
		src.print("throws ", false);
	    }
	    src.print(exceptions[e-1].getName(), false);
	    if (e < exceptions.length) {
		src.print(",", false);
	    }
	    src.print(" ", false);
	}
	src.println("{", false);
	// begining of the core
	src.indentPlus();
	src.println("return(findByPrimaryKey(("+ejbPrimaryKeyName+")p1));");
	src.indentMinus();
	src.println("}");
	src.println();
    }

}

