/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import org.objectweb.jonas.deployment.api.EntityCmpDesc;
import org.objectweb.jonas.deployment.api.EntityDesc;
import org.objectweb.jonas.lib.BeanNaming;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Entity Enterprise Java Bean.
 *  Its extends GenICRemote class.
 */

abstract class GenICEntityRemote extends GenICRemote {

    // Full name of the Handle class
    String wrpHandleName = null;
    // Full name of the PrimaryClass class
    String ejbPrimaryKeyName = null;

    /**
     * GenICEntityRemote Constructor
     *
     * @param dd       deployment descriptor of the Entity Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityRemote(EntityDesc dd, String fileName) throws GenICException {
	super(dd, fileName);

	BeanNaming bn = new BeanNaming(ddesc);
	wrpHandleName = bn.getFullWrpHandleName();
	ejbPrimaryKeyName = ((EntityDesc)ddesc).getPrimaryKeyClass().getName();
    }

    /**
     * Name of the superclass of the generated class
     * (JBeanEntity in case of Entity Bean)
     */
    String getSuperClass() {
	return(new String("JBeanEntity"));
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {
	src.println("public " + wrpRemoteBaseName +
		    "(JEntityHome home) throws RemoteException {");
	src.indentPlus();
	src.println("super(home,"+(ddesc instanceof EntityCmpDesc)+");");
	src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + "()\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation of the getHandle() method
     * @exception GenICException in error case
     */
    void genGetHandle() throws GenICException {
	src.println("public javax.ejb.Handle getHandle() throws RemoteException {");
	src.indentPlus();
	src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".getHandle()\");");
	src.println("if(nosuchobject){");
	src.indentPlus();
	src.println("throw new java.rmi.NoSuchObjectException(\"EJBObject already removed\");");
	src.indentMinus();
	src.println("}");
	src.println("if (handleEJBObject == null) {");
	src.indentPlus();
	src.println("handleEJBObject = new "+wrpHandleName+"(this);");
	src.indentMinus();
	src.println("}");
	src.println("return(handleEJBObject);");
	src.indentMinus();
	src.println("}");
    }

}

