/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.io.IOException;
import java.lang.String;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.IndentedFile;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.BeanDesc;
import org.objectweb.jonas.lib.BeanNaming;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Enterprise Java Bean.
 */
abstract class GenICHome extends GenICCommon {

    BeanDesc ddesc = null;

    // Full names of the interfaces and the bean classe
    String ejbRemoteName = null;
    String ejbHomeName   = null;
    String ejbBeanName   = null;
    // Classes of the interfaces and the bean
    Class  ejbRemoteClass = null;
    Class  ejbHomeClass   = null;
    Class  ejbBeanClass   = null;
    // Names of the implementation classes (Full names and base names)
    String wrpRemoteName     = null;
    String wrpHomeName       = null;
    String wrpRemoteBaseName = null;
    String wrpHomeBaseName   = null;

    /**
     * GenICHome Constructor
     *
     * @param dd       deployment descriptor of the Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICHome(BeanDesc dd, String fileName) throws GenICException {

	super(dd, fileName);

	// Deployment Descriptor
	ddesc = dd;
	// Names building
	ejbRemoteName = ddesc.getRemoteClass().getName();
	ejbHomeName   = ddesc.getHomeClass().getName();
	ejbBeanName   = ddesc.getEjbClass().getName();
	String sPack;
	BeanNaming bn     = new BeanNaming(ddesc);
	wrpRemoteBaseName = bn.getWrpRemoteName();
	wrpRemoteName     = bn.getFullWrpRemoteName();
	wrpHomeBaseName   = bn.getWrpHomeName();
	wrpHomeName       = bn.getFullWrpHomeName();
	// Get the interfaces and the bean classes
	ejbRemoteClass = ddesc.getRemoteClass();
	ejbHomeClass   = ddesc.getHomeClass();
	ejbBeanClass   = ddesc.getEjbClass();
    }

    /**
     * Generate the source of the class which implements the Home interface
     * @param withError the source file is generated with a syntax error
     *                  (internal use only for test)
     * @exception GenICException in error case
     */
    void generate(boolean withError) throws GenICException {

	// Declared methods in the Enterprise bean's home interface;
	Method methods[];

	// Comments
	src.println("//");
	src.println("// "+wrpHomeBaseName+".java");
	src.println("// DO NOT EDIT THIS FILE");
	src.println("// It is automatically generated via the GenIC tool of JOnAS");
	src.println("//");
	src.println();

	// Package clause
	String s = BeanNaming.getPackageName(ejbHomeName);
	if (s.length() != 0) {
	    src.println("package "+s+";");
	}
	src.println();

	// Wrong import clause generation for test only !!!!!
	if (withError) {
	    src.println("import java.toto.Titi;");
	    src.println();
	}

	// Import clauses
	genImport();
	src.println();

	// Class declaration
	src.println("public class "+wrpHomeBaseName+
		    " extends "+getSuperClass()+
		    " implements "+ejbHomeName+" {");
	src.println();
	src.indentPlus();

	// Constructor()
	genConstructor();
	src.println();

	// create() and find...() Methodes defined in the home interface
	methods = ejbHomeClass.getMethods();
	for (int m=0; m < methods.length; m++) {
	    // Skip the methods defined in the 'javax.ejb.EJBHome' interface
	    Method method = methods[m];
	    if (!method.getDeclaringClass().getName().equals("javax.ejb.EJBHome")) {

		boolean isCreate    = method.getName().equals("create");
		boolean isFindByPK  = method.getName().equals("findByPrimaryKey");
		boolean isFindOther = ((!isFindByPK) && (method.getName().indexOf("find") == 0));

		if (isCreate) {
		    genCreate(method);
		} else if (isFindByPK) {
		    genFindByPK(method);
		} else if (isFindOther) {
		    genFindOther(method);
		} else {
		    Trace.outln("GenICHome() WARNING: Method '" + method.getName() +
				"' in the Enterprise bean's home interface IGNORED");
		}

		src.println();
	    }
	}

	// Class End
	src.indentMinus();
	src.println("}");
	src.println();

	//
	src.close();
    }

    /**
     * Source generation of the header of the given Method.
     * (Declaration and the begining of the core..)
     * @exception GenICException in error case
     */
    void genHeaderMethod(Method method) throws GenICException {

	Class paramsTable[]     = method.getParameterTypes();
	Class exceptionsTable[] = method.getExceptionTypes();

	// declaration
	src.print("synchronized public " +
		  JavaType.getName(method.getReturnType()) +
		  " " +
		  method.getName() +
		  "(");
	for (int p=1; p <= paramsTable.length; p++) {
	    src.print(JavaType.getName(paramsTable[p-1]) + " " + "p" + p, false);
	    if (p < paramsTable.length) {
		src.print(", ", false);
	    }
	}
	src.print(") ", false);
	for (int e=1; e <= exceptionsTable.length; e++) {
	    if (e==1) {
		src.print("throws ", false);
	    }
	    src.print(exceptionsTable[e-1].getName(), false);
	    if (e < exceptionsTable.length) {
		src.print(",", false);
	    }
	    src.print(" ", false);
	}
	src.println("{", false);
	// begining of the core
	src.indentPlus();
	src.print("Trace.outln(Trace.DB_15, \""+wrpHomeBaseName+"."+method.getName()+"(");
	for (int p=1; p <= paramsTable.length; p++) {
	    src.print(JavaType.getName(paramsTable[p-1]), false);
	    if (p < paramsTable.length) {
		src.print(", ", false);
	    }
	}
	src.println(")\");", false);
    }

    /**
     * Name of the superclass of the generated class
     * (JSessionHome or JEntityHome, depending on the Bean type)
     */
    abstract String getSuperClass();

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void   genImport() throws GenICException;

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    abstract void   genConstructor() throws GenICException;

    /**
     * Source generation for the given create() method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genCreate(Method method) throws GenICException;

    /**
     * Source generation for the given findByPrimaryKey() method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genFindByPK(Method method) throws GenICException;

    /**
     * Source generation for the given findXXX() method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genFindOther(Method method) throws GenICException;

}

