/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */



package org.objectweb.jonas.tools;


import java.io.IOException;
import java.lang.String;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.IndentedFile;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.BeanDesc;
import org.objectweb.jonas.deployment.api.SessionDesc;
import org.objectweb.jonas.lib.BeanNaming;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Enterprise Java Bean.
 */

abstract class GenICRemote extends GenICCommon {

    BeanDesc     ddesc = null;

    // Full names of the interfaces and the bean classe
    String ejbRemoteName     = null;
    String ejbBeanName       = null;
    // Classes of the interfaces and the bean
    Class  ejbRemoteClass = null;
    Class  ejbBeanClass   = null;
    // Names of the implementation classes (Full names and base names)
    String wrpRemoteName     = null;
    String wrpRemoteBaseName = null;


    /**
     * GenICRemote Constructor
     *
     * @param dd       deployment descriptor of the Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICRemote(BeanDesc dd, String fileName) throws GenICException {

	super(dd, fileName);

	// Deployment Descriptor
	ddesc = dd;
	// Names building
	ejbRemoteName = ddesc.getRemoteClass().getName();
	ejbBeanName = ddesc.getEjbClass().getName();
	String sPack;
	BeanNaming bn     = new BeanNaming(ddesc);
	wrpRemoteBaseName = bn.getWrpRemoteName();
	wrpRemoteName     = bn.getFullWrpRemoteName();
	// Get the interfaces and the bean classes
	ejbRemoteClass = ddesc.getRemoteClass();
	ejbBeanClass   = ddesc.getEjbClass();

    }

    /**
     * Generate the source of the class which implements the Remote interface
     * @param withError the source file is generated with a syntax error
     *                  (internal use only for test)
     * @exception GenICException in error case
     */
    void generate(boolean withError) throws GenICException {

	// Declared methods in the Enterprise bean's remote interface;
	Method methods[];

	// Comments
	src.println("//");
	src.println("// "+wrpRemoteBaseName+".java");
	src.println("// DO NOT EDIT THIS FILE");
	src.println("// It is automatically generated via the GenIC tool of JOnAS");
	src.println("//");
	src.println();

	// Package clause
	String s = BeanNaming.getPackageName(ejbRemoteName);
	if (s.length() != 0) {
	    src.println("package "+s+";");
	}
	src.println();

	// Wrong import clause generation for test only !!!!!
	if (withError) {
	    src.println("import java.toto.Titi;");
	    src.println();
	}

	// Import clauses
	genImport();
	src.println();

	// Class declaration
	src.println("public class "+wrpRemoteBaseName+
		    " extends "+getSuperClass()+
		    " implements "+ejbRemoteName+" {");
	src.println();
	src.indentPlus();

	// Constructor()
	genConstructor();
	src.println();

	// getHandle() Method
	genGetHandle();
	src.println();

	// Bussiness Methodes defined in the remote interface
	methods = ejbRemoteClass.getMethods();
	for (int m=0; m < methods.length; m++) {
	    // Skip the methods defined in the 'javax.ejb.EJBObject' interface
	    Method method = methods[m];
	    if (!method.getDeclaringClass().getName().equals("javax.ejb.EJBObject")) {
		genBussinessMethod(method);
		src.println();
	    }
	}

	// createData(), removeData(), loadData(), storeData()
	genAccessData();
	src.println();

	// End of the Class
	src.indentMinus();
	src.println("}");
	src.println();

	//
	src.close();
    }

    /**
     * Source generation for the given bussiness method described in the Remote interface
     * @exception GenICException in error case
     */
    void genBussinessMethod(Method method) throws GenICException {

	int transAttribute;
	Class params[] = method.getParameterTypes();
	Class exceptions[] = method.getExceptionTypes();
	boolean isReturnValue = ! JavaType.isVoid(method.getReturnType());

	// declaration
	src.print("public ");
	if (ddesc instanceof SessionDesc) {
	    src.print("synchronized ", false);
	}
	src.print(JavaType.getName(method.getReturnType()) + " " +
		  method.getName() +
		  "("
		  , false);

	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]) + " " + "p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.print(") ", false);
	for (int e=1; e <= exceptions.length; e++) {
	    if (e==1) {
		src.print("throws ", false);
	    }
	    src.print(exceptions[e-1].getName(), false);
	    if (e < exceptions.length) {
		src.print(",", false);
	    }
	    src.print(" ", false);
	}
	src.println("{", false);

	// core
	src.indentPlus();
	src.println("Trace.outln(Trace.DB_15, \"" +
		    wrpRemoteBaseName + "." + method.getName() +
		    "()\");");
	transAttribute = ddesc.getMethodDesc(method).getTxAttribute();
	src.println("RequestCtx rctx = preinvoke(" + transAttribute + ");");
	if (ddesc instanceof SessionDesc) {
	    src.println("SessionContextImpl ctx = (SessionContextImpl) rctx.getEJBContext();");
	} else {
	    src.println("EntityContextImpl ctx = (EntityContextImpl) rctx.getEJBContext();");
	}
	src.println(ejbBeanName + " eb = ("+ejbBeanName +")ctx.getInstance();");
	if (isReturnValue) {
	    src.println(JavaType.getName(method.getReturnType()) + " result;");
	}
	src.println("try {");
	src.indentPlus();
	if (isReturnValue) {
	    src.print("result = ");
	}
	src.print("eb." + method.getName() + "(", !isReturnValue);
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	genCatchBean(method);
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	if (isReturnValue) {
	    src.println("return(result);");
	}
	src.indentMinus(); // end of core
	src.println("}");
    }

    /**
     * Name of the superclass of the generated class
     * (JBeanSession or JBeanEntity, depending on the Bean type)
     */
    abstract String getSuperClass();

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void   genImport() throws GenICException;

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    abstract void   genConstructor() throws GenICException;

    /**
     * Source generation of the getHandle() method
     * (In some case, no source is generated; see sub-classes)
     * @exception GenICException in error case
     */
    abstract void   genGetHandle() throws GenICException;

    /**
     * Source generation of the createData(), removeData(), loadData(), storeData() methods
     * (In some case, no source is generated; see sub-classes)
     * @exception GenICException in error case
     */
    abstract void   genAccessData() throws GenICException;

}

