/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import org.objectweb.jonas.deployment.api.SessionDesc;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Session Enterprise Java Bean.
 *  Its extends GenICRemote class.
 */

class GenICSessionRemote extends GenICRemote {

    boolean isSynchro = false;

    /**
     * GenICSessionRemote Constructor
     *
     * @param dd       deployment descriptor of the Session Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICSessionRemote(SessionDesc dd, String fileName) throws GenICException {
	super(dd, fileName);

	// Is the Session Bean a SessionSynchronization ?
	isSynchro = JavaType.implementsInterface(ejbBeanClass,
						 javax.ejb.SessionSynchronization.class); 
    }

    /**
     * Name of the superclass of the generated class
     * (JBeanSession in case of Session Bean)
     */
    String getSuperClass() {
	return(new String("JBeanSession"));
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import javax.ejb.EnterpriseBean;");
	src.println("import javax.ejb.RemoveException;");
	src.println("import javax.ejb.SessionBean;");
	src.println("import org.objectweb.jonas.container.JSessionHome;");
	src.println("import org.objectweb.jonas.container.JBeanSession;");
	src.println("import org.objectweb.jonas.container.RequestCtx;");
	src.println("import org.objectweb.jonas.container.SessionContextImpl;");
	src.println("import org.objectweb.jonas.container.SessionSynchroImpl;");
	src.println("import org.objectweb.jonas.container.SynchroImpl;");
	src.println("import org.objectweb.jonas.common.Trace;");
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {
	src.println("public " + wrpRemoteBaseName +
		    "(JSessionHome home) throws RemoteException {");
	src.indentPlus();
	src.println("super(home,"+isSynchro+");");
	src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + "()\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation of the getHandle() method
     * In case of Session Bean, there is no source to generate.
     * @exception GenICException in error case
     */
    void genGetHandle() throws GenICException {
	// Nothing to do !!!
	// No source to generate for Session (Generic source in SessionHandleImpl)
    }

    /**
     * Source generation of the createData(), removeData(), loadData(), storeData() methods
     * In case of Session Bean, there is no source to generate.
     * @exception GenICException in error case
     */
    void genAccessData() throws GenICException {
	// Nothing to do !!!
	// No source to generate for Session (No persistence container managed)
    }

}

